@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2025 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -41,10 +41,10 @@ BEGIN_ROCPRIM_NAMESPACE
 /// \brief The \p block_exchange class is a block level parallel primitive which provides
 /// methods for rearranging items partitioned across threads in a block.
 ///
-/// \tparam T - the input type.
-/// \tparam BlockSize - the number of threads in a block.
-/// \tparam ItemsPerThread - the number of items contributed by each thread.
-/// \tparam PaddingHint - a hint that decides when to use padding. May not always be applicable.
+/// \tparam T the input type.
+/// \tparam BlockSize the number of threads in a block.
+/// \tparam ItemsPerThread the number of items contributed by each thread.
+/// \tparam PaddingHint a hint that decides when to use padding. May not always be applicable.
 ///
 /// \par Overview
 /// * The \p block_exchange class supports the following rearrangement methods:
@@ -77,20 +77,20 @@ BEGIN_ROCPRIM_NAMESPACE
 /// }
 /// \endcode
 /// \endparblock
-template<
-    class T,
-    unsigned int BlockSizeX,
-    unsigned int ItemsPerThread,
-    unsigned int BlockSizeY = 1,
-    unsigned int BlockSizeZ = 1,
-    block_padding_hint PaddingHint = block_padding_hint::avoid_conflicts
->
+template<class T,
+         unsigned int                       BlockSizeX,
+         unsigned int                       ItemsPerThread,
+         unsigned int                       BlockSizeY  = 1,
+         unsigned int                       BlockSizeZ  = 1,
+         block_padding_hint                 PaddingHint = block_padding_hint::avoid_conflicts,
+         ::rocprim::arch::wavefront::target TargetWaveSize
+         = ::rocprim::arch::wavefront::get_target()>
 class block_exchange
 {
     static constexpr unsigned int BlockSize = BlockSizeX * BlockSizeY * BlockSizeZ;
     // Select warp size
-    static constexpr unsigned int warp_size =
-        detail::get_min_warp_size(BlockSize, ::rocprim::arch::wavefront::min_size());
+    static constexpr unsigned int warp_size = ::rocprim::detail::get_min_warp_size(
+        BlockSize, ::rocprim::arch::wavefront::size_from_target<TargetWaveSize>());
     // Number of warps in block
     static constexpr unsigned int warps_no = ::rocprim::detail::ceiling_div(BlockSize, warp_size);
     static constexpr unsigned int banks_no = ::rocprim::detail::get_lds_banks_no();
@@ -149,10 +149,10 @@ public:
     /// \brief Transposes a blocked arrangement of items to a striped arrangement
     /// across the thread block.
     ///
-    /// \tparam U - [inferred] the output type.
+    /// \tparam U [inferred] the output type.
     ///
-    /// \param [in] input - array that data is loaded from.
-    /// \param [out] output - array that data is loaded to.
+    /// \param [in] input array that data is loaded from.
+    /// \param [out] output array that data is loaded to.
     template<class U>
     ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE
     void blocked_to_striped(const T (&input)[ItemsPerThread],
@@ -165,11 +165,11 @@ public:
     /// \brief Transposes a blocked arrangement of items to a striped arrangement
     /// across the thread block, using temporary storage.
     ///
-    /// \tparam U - [inferred] the output type.
+    /// \tparam U [inferred] the output type.
     ///
-    /// \param [in] input - array that data is loaded from.
-    /// \param [out] output - array that data is loaded to.
-    /// \param [in] storage - reference to a temporary storage object of type storage_type.
+    /// \param [in] input array that data is loaded from.
+    /// \param [out] output array that data is loaded to.
+    /// \param [in] storage reference to a temporary storage object of type storage_type.
     ///
     /// \par Storage reusage
     /// Synchronization barrier should be placed before \p storage is reused
@@ -216,10 +216,10 @@ public:
     /// \brief Transposes a striped arrangement of items to a blocked arrangement
     /// across the thread block.
     ///
-    /// \tparam U - [inferred] the output type.
+    /// \tparam U [inferred] the output type.
     ///
-    /// \param [in] input - array that data is loaded from.
-    /// \param [out] output - array that data is loaded to.
+    /// \param [in] input array that data is loaded from.
+    /// \param [out] output array that data is loaded to.
     template<class U>
     ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE
     void striped_to_blocked(const T (&input)[ItemsPerThread],
@@ -232,11 +232,11 @@ public:
     /// \brief Transposes a striped arrangement of items to a blocked arrangement
     /// across the thread block, using temporary storage.
     ///
-    /// \tparam U - [inferred] the output type.
+    /// \tparam U [inferred] the output type.
     ///
-    /// \param [in] input - array that data is loaded from.
-    /// \param [out] output - array that data is loaded to.
-    /// \param [in] storage - reference to a temporary storage object of type storage_type.
+    /// \param [in] input array that data is loaded from.
+    /// \param [out] output array that data is loaded to.
+    /// \param [in] storage reference to a temporary storage object of type storage_type.
     ///
     /// \par Storage reusage
     /// Synchronization barrier should be placed before \p storage is reused
@@ -283,10 +283,10 @@ public:
     /// \brief Transposes a blocked arrangement of items to a warp-striped arrangement
     /// across the thread block.
     ///
-    /// \tparam U - [inferred] the output type.
+    /// \tparam U [inferred] the output type.
     ///
-    /// \param [in] input - array that data is loaded from.
-    /// \param [out] output - array that data is loaded to.
+    /// \param [in] input array that data is loaded from.
+    /// \param [out] output array that data is loaded to.
     template<class U>
     ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE
     void blocked_to_warp_striped(const T (&input)[ItemsPerThread],
@@ -299,11 +299,11 @@ public:
     /// \brief Transposes a blocked arrangement of items to a warp-striped arrangement
     /// across the thread block, using temporary storage.
     ///
-    /// \tparam U - [inferred] the output type.
+    /// \tparam U [inferred] the output type.
     ///
-    /// \param [in] input - array that data is loaded from.
-    /// \param [out] output - array that data is loaded to.
-    /// \param [in] storage - reference to a temporary storage object of type storage_type.
+    /// \param [in] input array that data is loaded from.
+    /// \param [out] output array that data is loaded to.
+    /// \param [in] storage reference to a temporary storage object of type storage_type.
     ///
     /// \par Storage reusage
     /// Synchronization barrier should be placed before \p storage is reused
@@ -354,10 +354,10 @@ public:
     /// \brief Transposes a warp-striped arrangement of items to a blocked arrangement
     /// across the thread block.
     ///
-    /// \tparam U - [inferred] the output type.
+    /// \tparam U [inferred] the output type.
     ///
-    /// \param [in] input - array that data is loaded from.
-    /// \param [out] output - array that data is loaded to.
+    /// \param [in] input array that data is loaded from.
+    /// \param [out] output array that data is loaded to.
     template<class U>
     ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE
     void warp_striped_to_blocked(const T (&input)[ItemsPerThread],
@@ -370,11 +370,11 @@ public:
     /// \brief Transposes a warp-striped arrangement of items to a blocked arrangement
     /// across the thread block, using temporary storage.
     ///
-    /// \tparam U - [inferred] the output type.
+    /// \tparam U [inferred] the output type.
     ///
-    /// \param [in] input - array that data is loaded from.
-    /// \param [out] output - array that data is loaded to.
-    /// \param [in] storage - reference to a temporary storage object of type storage_type.
+    /// \param [in] input array that data is loaded from.
+    /// \param [out] output array that data is loaded to.
+    /// \param [in] storage reference to a temporary storage object of type storage_type.
     ///
     /// \par Storage reusage
     /// Synchronization barrier should be placed before \p storage is reused
@@ -425,12 +425,12 @@ public:
     /// \brief Scatters items to a blocked arrangement based on their ranks
     /// across the thread block.
     ///
-    /// \tparam U - [inferred] the output type.
-    /// \tparam Offset - [inferred] the rank type.
+    /// \tparam U [inferred] the output type.
+    /// \tparam Offset [inferred] the rank type.
     ///
-    /// \param [in] input - array that data is loaded from.
-    /// \param [out] output - array that data is loaded to.
-    /// \param [out] ranks - array that has rank of data.
+    /// \param [in] input array that data is loaded from.
+    /// \param [out] output array that data is loaded to.
+    /// \param [out] ranks array that has rank of data.
     template<class U, class Offset>
     ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE
     void scatter_to_blocked(const T (&input)[ItemsPerThread],
@@ -444,12 +444,12 @@ public:
     /// \brief Gathers items from a striped arrangement based on their ranks
     /// across the thread block.
     ///
-    /// \tparam U - [inferred] the output type.
-    /// \tparam Offset - [inferred] the rank type.
+    /// \tparam U [inferred] the output type.
+    /// \tparam Offset [inferred] the rank type.
     ///
-    /// \param [in] input - array that data is loaded from.
-    /// \param [out] output - array that data is loaded to.
-    /// \param [out] ranks - array that has rank of data.
+    /// \param [in] input array that data is loaded from.
+    /// \param [out] output array that data is loaded to.
+    /// \param [out] ranks array that has rank of data.
     template<class U, class Offset>
     ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE
     void gather_from_striped(const T (&input)[ItemsPerThread],
@@ -463,13 +463,13 @@ public:
     /// \brief Scatters items to a blocked arrangement based on their ranks
     /// across the thread block, using temporary storage.
     ///
-    /// \tparam U - [inferred] the output type.
-    /// \tparam Offset - [inferred] the rank type.
+    /// \tparam U [inferred] the output type.
+    /// \tparam Offset [inferred] the rank type.
     ///
-    /// \param [in] input - array that data is loaded from.
-    /// \param [out] output - array that data is loaded to.
-    /// \param [out] ranks - array that has rank of data.
-    /// \param [in] storage - reference to a temporary storage object of type storage_type.
+    /// \param [in] input array that data is loaded from.
+    /// \param [out] output array that data is loaded to.
+    /// \param [out] ranks array that has rank of data.
+    /// \param [in] storage reference to a temporary storage object of type storage_type.
     ///
     /// \par Storage reusage
     /// Synchronization barrier should be placed before \p storage is reused
@@ -519,13 +519,13 @@ public:
     /// \brief Gathers items from a striped arrangement based on their ranks
     /// across the thread block, using temporary storage.
     ///
-    /// \tparam U - [inferred] the output type.
-    /// \tparam Offset - [inferred] the rank type.
+    /// \tparam U [inferred] the output type.
+    /// \tparam Offset [inferred] the rank type.
     ///
-    /// \param [in] input - array that data is loaded from.
-    /// \param [out] output - array that data is loaded to.
-    /// \param [out] ranks - array that has rank of data.
-    /// \param [in] storage - reference to a temporary storage object of type storage_type.
+    /// \param [in] input array that data is loaded from.
+    /// \param [out] output array that data is loaded to.
+    /// \param [out] ranks array that has rank of data.
+    /// \param [in] storage reference to a temporary storage object of type storage_type.
     template <class U, class Offset>
     ROCPRIM_DEVICE ROCPRIM_INLINE
     void gather_from_striped(const T (&input)[ItemsPerThread],
@@ -553,12 +553,12 @@ public:
     /// \brief Scatters items to a striped arrangement based on their ranks
     /// across the thread block.
     ///
-    /// \tparam U - [inferred] the output type.
-    /// \tparam Offset - [inferred] the rank type.
+    /// \tparam U [inferred] the output type.
+    /// \tparam Offset [inferred] the rank type.
     ///
-    /// \param [in] input - array that data is loaded from.
-    /// \param [out] output - array that data is loaded to.
-    /// \param [out] ranks - array that has rank of data.
+    /// \param [in] input array that data is loaded from.
+    /// \param [out] output array that data is loaded to.
+    /// \param [out] ranks array that has rank of data.
     template<class U, class Offset>
     ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE
     void scatter_to_striped(const T (&input)[ItemsPerThread],
@@ -572,13 +572,13 @@ public:
     /// \brief Scatters items to a striped arrangement based on their ranks
     /// across the thread block, using temporary storage.
     ///
-    /// \tparam U - [inferred] the output type.
-    /// \tparam Offset - [inferred] the rank type.
+    /// \tparam U [inferred] the output type.
+    /// \tparam Offset [inferred] the rank type.
     ///
-    /// \param [in] input - array that data is loaded from.
-    /// \param [out] output - array that data is loaded to.
-    /// \param [out] ranks - array that has rank of data.
-    /// \param [in] storage - reference to a temporary storage object of type storage_type.
+    /// \param [in] input array that data is loaded from.
+    /// \param [out] output array that data is loaded to.
+    /// \param [out] ranks array that has rank of data.
+    /// \param [in] storage reference to a temporary storage object of type storage_type.
     ///
     /// \par Storage reusage
     /// Synchronization barrier should be placed before \p storage is reused
@@ -628,13 +628,13 @@ public:
     /// \brief Scatters items to a *warp* striped arrangement based on their ranks
     /// across the thread block, using temporary storage.
     ///
-    /// \tparam U - [inferred] the output type.
-    /// \tparam Offset - [inferred] the rank type.
+    /// \tparam U [inferred] the output type.
+    /// \tparam Offset [inferred] the rank type.
     ///
-    /// \param [in] input - array that data is loaded from.
-    /// \param [out] output - array that data is loaded to.
-    /// \param [out] ranks - array that has rank of data.
-    /// \param [in] storage - reference to a temporary storage object of type storage_type.
+    /// \param [in] input array that data is loaded from.
+    /// \param [out] output array that data is loaded to.
+    /// \param [out] ranks array that has rank of data.
+    /// \param [in] storage reference to a temporary storage object of type storage_type.
     ///
     /// \par Storage reusage
     /// Synchronization barrier should be placed before \p storage is reused
@@ -657,23 +657,26 @@ public:
     ///     ...
     /// }
     /// \endcode
-    template<unsigned int WarpSize = arch::wavefront::min_size(), class U, class Offset>
+    template<unsigned int VirtualWaveSize = arch::wavefront::size_from_target<TargetWaveSize>(),
+             class U,
+             class Offset>
     ROCPRIM_DEVICE ROCPRIM_INLINE
     void scatter_to_warp_striped(const T (&input)[ItemsPerThread],
                                  U (&output)[ItemsPerThread],
                                  const Offset (&ranks)[ItemsPerThread],
                                  storage_type& storage)
     {
-        static_assert(detail::is_power_of_two(WarpSize) && WarpSize <= arch::wavefront::max_size(),
-                      "WarpSize must be a power of two and equal or less"
+        static_assert(detail::is_power_of_two(VirtualWaveSize)
+                          && VirtualWaveSize <= arch::wavefront::max_size(),
+                      "VirtualWaveSize must be a power of two and equal or less "
                       "than the size of hardware warp.");
-        assert(WarpSize <= arch::wavefront::size());
-        
+        assert(VirtualWaveSize <= arch::wavefront::size());
+
         const unsigned int flat_id
             = ::rocprim::flat_block_thread_id<BlockSizeX, BlockSizeY, BlockSizeZ>();
-        const unsigned int thread_id     = detail::logical_lane_id<WarpSize>();
-        const unsigned int warp_id       = flat_id / WarpSize;
-        const unsigned int warp_offset   = warp_id * WarpSize * ItemsPerThread;
+        const unsigned int thread_id     = detail::logical_lane_id<VirtualWaveSize>();
+        const unsigned int warp_id       = flat_id / VirtualWaveSize;
+        const unsigned int warp_offset   = warp_id * VirtualWaveSize * ItemsPerThread;
         const unsigned int thread_offset = thread_id + warp_offset;
 
         ROCPRIM_UNROLL
@@ -690,7 +693,7 @@ public:
         ROCPRIM_UNROLL
         for(unsigned int i = 0; i < ItemsPerThread; i++)
         {
-            output[i] = storage_buffer[index(thread_offset + i * WarpSize)];
+            output[i] = storage_buffer[index(thread_offset + i * VirtualWaveSize)];
         }
     }
 
@@ -700,12 +703,12 @@ public:
     /// \par Overview
     /// * Items with rank -1 are not scattered.
     ///
-    /// \tparam U - [inferred] the output type.
-    /// \tparam Offset - [inferred] the rank type.
+    /// \tparam U [inferred] the output type.
+    /// \tparam Offset [inferred] the rank type.
     ///
-    /// \param [in] input - array that data is loaded from.
-    /// \param [out] output - array that data is loaded to.
-    /// \param [in] ranks - array that has rank of data.
+    /// \param [in] input array that data is loaded from.
+    /// \param [out] output array that data is loaded to.
+    /// \param [in] ranks array that has rank of data.
     template<class U, class Offset>
     ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE
     void scatter_to_striped_guarded(const T (&input)[ItemsPerThread],
@@ -722,13 +725,13 @@ public:
     /// \par Overview
     /// * Items with rank -1 are not scattered.
     ///
-    /// \tparam U - [inferred] the output type.
-    /// \tparam Offset - [inferred] the rank type.
+    /// \tparam U [inferred] the output type.
+    /// \tparam Offset [inferred] the rank type.
     ///
-    /// \param [in] input - array that data is loaded from.
-    /// \param [out] output - array that data is loaded to.
-    /// \param [in] ranks - array that has rank of data.
-    /// \param [in] storage - reference to a temporary storage object of type storage_type.
+    /// \param [in] input array that data is loaded from.
+    /// \param [out] output array that data is loaded to.
+    /// \param [in] ranks array that has rank of data.
+    /// \param [in] storage reference to a temporary storage object of type storage_type.
     ///
     /// \par Storage reusage
     /// Synchronization barrier should be placed before \p storage is reused
@@ -781,14 +784,14 @@ public:
     /// \brief Scatters items to a striped arrangement based on their ranks
     /// across the thread block, with a flag to denote validity.
     ///
-    /// \tparam U - [inferred] the output type.
-    /// \tparam Offset - [inferred] the rank type.
-    /// \tparam ValidFlag - [inferred] the validity flag type.
+    /// \tparam U [inferred] the output type.
+    /// \tparam Offset [inferred] the rank type.
+    /// \tparam ValidFlag [inferred] the validity flag type.
     ///
-    /// \param [in] input - array that data is loaded from.
-    /// \param [out] output - array that data is loaded to.
-    /// \param [in] ranks - array that has rank of data.
-    /// \param [in] is_valid - array that has flags to denote validity.
+    /// \param [in] input array that data is loaded from.
+    /// \param [out] output array that data is loaded to.
+    /// \param [in] ranks array that has rank of data.
+    /// \param [in] is_valid array that has flags to denote validity.
     template<class U, class Offset, class ValidFlag>
     ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE
     void scatter_to_striped_flagged(const T (&input)[ItemsPerThread],
@@ -804,15 +807,15 @@ public:
     /// across the thread block, with a flag to denote validity, using temporary
     /// storage.
     ///
-    /// \tparam U - [inferred] the output type.
-    /// \tparam Offset - [inferred] the rank type.
-    /// \tparam ValidFlag - [inferred] the validity flag type.
+    /// \tparam U [inferred] the output type.
+    /// \tparam Offset [inferred] the rank type.
+    /// \tparam ValidFlag [inferred] the validity flag type.
     ///
-    /// \param [in] input - array that data is loaded from.
-    /// \param [out] output - array that data is loaded to.
-    /// \param [in] ranks - array that has rank of data.
-    /// \param [in] is_valid - array that has flags to denote validity.
-    /// \param [in] storage - reference to a temporary storage object of type storage_type.
+    /// \param [in] input array that data is loaded from.
+    /// \param [out] output array that data is loaded to.
+    /// \param [in] ranks array that has rank of data.
+    /// \param [in] is_valid array that has flags to denote validity.
+    /// \param [in] storage reference to a temporary storage object of type storage_type.
     ///
     /// \par Storage reusage
     /// Synchronization barrier should be placed before \p storage is reused
@@ -884,6 +887,121 @@ private:
     }
 };
 
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
+
+template<typename T,
+         unsigned int       BlockSizeX,
+         unsigned int       ItemsPerThread,
+         unsigned int       BlockSizeY,
+         unsigned int       BlockSizeZ,
+         block_padding_hint PaddingHint>
+class block_exchange<T,
+                     BlockSizeX,
+                     ItemsPerThread,
+                     BlockSizeY,
+                     BlockSizeZ,
+                     PaddingHint,
+                     ::rocprim::arch::wavefront::target::dynamic>
+{
+private:
+    using block_exchange_wave32 = block_exchange<T,
+                                                 BlockSizeX,
+                                                 ItemsPerThread,
+                                                 BlockSizeY,
+                                                 BlockSizeZ,
+                                                 PaddingHint,
+                                                 ::rocprim::arch::wavefront::target::size32>;
+    using block_exchange_wave64 = block_exchange<T,
+                                                 BlockSizeX,
+                                                 ItemsPerThread,
+                                                 BlockSizeY,
+                                                 BlockSizeZ,
+                                                 PaddingHint,
+                                                 ::rocprim::arch::wavefront::target::size64>;
+    using dispatch
+        = ::rocprim::detail::dispatch_wave_size<block_exchange_wave32, block_exchange_wave64>;
+
+public:
+    using storage_type = typename dispatch::storage_type;
+
+    template<typename... Args>
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    auto blocked_to_striped(Args&&... args)
+    {
+        dispatch{}([](auto impl, auto&&... args) { impl.blocked_to_striped(args...); }, args...);
+    }
+
+    template<typename... Args>
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    auto striped_to_blocked(Args&&... args)
+    {
+        dispatch{}([](auto impl, auto&&... args) { impl.striped_to_blocked(args...); }, args...);
+    }
+
+    template<typename... Args>
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    auto blocked_to_warp_striped(Args&&... args)
+    {
+        dispatch{}([](auto impl, auto&&... args) { impl.blocked_to_warp_striped(args...); },
+                   args...);
+    }
+
+    template<typename... Args>
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    auto warp_striped_to_blocked(Args&&... args)
+    {
+        dispatch{}([](auto impl, auto&&... args) { impl.warp_striped_to_blocked(args...); },
+                   args...);
+    }
+
+    template<typename... Args>
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    auto scatter_to_blocked(Args&&... args)
+    {
+        dispatch{}([](auto impl, auto&&... args) { impl.scatter_to_blocked(args...); }, args...);
+    }
+
+    template<typename... Args>
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    auto gather_from_striped(Args&&... args)
+    {
+        dispatch{}([](auto impl, auto&&... args) { impl.gather_from_striped(args...); }, args...);
+    }
+
+    template<typename... Args>
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    auto scatter_to_striped(Args&&... args)
+    {
+        dispatch{}([](auto impl, auto&&... args) { impl.scatter_to_striped(args...); }, args...);
+    }
+
+    template<typename... Args>
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    auto scatter_to_warp_striped(Args&&... args)
+    {
+        dispatch{}([](auto impl, auto&&... args) { impl.scatter_to_warp_striped(args...); },
+                   args...);
+    }
+
+    template<typename... Args>
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    auto scatter_to_striped_guarded(Args&&... args)
+    {
+        dispatch{}([](auto impl, auto&&... args) { impl.scatter_to_striped_guarded(args...); },
+                   args...);
+    }
+
+    template<typename... Args>
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    auto scatter_to_striped_flagged(Args&&... args)
+    {
+        dispatch{}([](auto impl, auto&&... args) { impl.scatter_to_striped_flagged(args...); },
+                   args...);
+    }
+};
+
+#endif
+
 END_ROCPRIM_NAMESPACE
 
 /// @}
