@@ -26,21 +26,21 @@
 #include "../../functional.hpp"
 #include "../../types.hpp"
 
-#include "../../thread/radix_key_codec.hpp"
-
 #include "../block_scan.hpp"
 #include "../config.hpp"
+#include "rocprim/intrinsics/arch.hpp"
 
 BEGIN_ROCPRIM_NAMESPACE
 
 namespace detail
 {
 
-template<unsigned int       BlockSizeX,
-         unsigned int       RadixBits,
-         unsigned int       BlockSizeY  = 1,
-         unsigned int       BlockSizeZ  = 1,
-         block_padding_hint PaddingHint = block_padding_hint::avoid_conflicts>
+template<unsigned int            BlockSizeX,
+         unsigned int            RadixBits,
+         unsigned int            BlockSizeY     = 1,
+         unsigned int            BlockSizeZ     = 1,
+         block_padding_hint      PaddingHint    = block_padding_hint::avoid_conflicts,
+         arch::wavefront::target TargetWaveSize = arch::wavefront::get_target()>
 class block_radix_rank_match
 {
     using digit_counter_type = unsigned int;
@@ -49,15 +49,17 @@ class block_radix_rank_match
                                                   BlockSizeX,
                                                   ::rocprim::block_scan_algorithm::using_warp_scan,
                                                   BlockSizeY,
-                                                  BlockSizeZ>;
+                                                  BlockSizeZ,
+                                                  TargetWaveSize>;
 
     static constexpr unsigned int block_size   = BlockSizeX * BlockSizeY * BlockSizeZ;
     static constexpr unsigned int radix_digits = 1 << RadixBits;
 
     struct unpadded_config
     {
-        static constexpr unsigned int warps
-            = ::rocprim::detail::ceiling_div(block_size, device_warp_size());
+        // min size is used because we allocate based on the number of warps
+        static constexpr unsigned int warps = ::rocprim::detail::ceiling_div(
+            block_size, arch::wavefront::size_from_target<TargetWaveSize>());
     };
 
     struct padded_config
@@ -202,7 +204,8 @@ private:
                                        const unsigned int begin_bit,
                                        const unsigned int pass_bits)
     {
-        using key_codec    = ::rocprim::radix_key_codec<Key, Descending>;
+        using key_codec
+            = decltype(::rocprim::traits::get<Key>().template radix_key_codec<Descending>());
         using bit_key_type = typename key_codec::bit_key_type;
 
         bit_key_type bit_keys[ItemsPerThread];
