@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2025 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -30,6 +30,7 @@
 #include "../../intrinsics.hpp"
 
 #include "../../warp/warp_reduce.hpp"
+#include "rocprim/intrinsics/arch.hpp"
 
 BEGIN_ROCPRIM_NAMESPACE
 
@@ -99,10 +100,11 @@ private:
 #endif // DOXYGEN_SHOULD_SKIP_THIS
 
 template<class T,
-         unsigned int BlockSizeX,
-         unsigned int BlockSizeY,
-         unsigned int BlockSizeZ,
-         bool         CommutativeOnly = false>
+         unsigned int            BlockSizeX,
+         unsigned int            BlockSizeY,
+         unsigned int            BlockSizeZ,
+         arch::wavefront::target TargetWaveSize,
+         bool                    CommutativeOnly = false>
 class block_reduce_raking_reduce
 {
     static constexpr unsigned int BlockSize = BlockSizeX * BlockSizeY * BlockSizeZ;
@@ -110,7 +112,7 @@ class block_reduce_raking_reduce
     // Warp reduce, warp_reduce_crosslane does not require shared memory (storage), but
     // logical warp size must be a power of two.
     static constexpr unsigned int warp_size_
-        = detail::get_min_warp_size(BlockSize, ::rocprim::device_warp_size());
+        = detail::get_min_warp_size(BlockSize, arch::wavefront::size_from_target<TargetWaveSize>());
 
     static constexpr unsigned int segment_len = ceiling_div(BlockSize, warp_size_);
 
