@@ -21,8 +21,8 @@
 #ifndef ROCPRIM_DEVICE_DETAIL_DEVICE_RADIX_SORT_HPP_
 #define ROCPRIM_DEVICE_DETAIL_DEVICE_RADIX_SORT_HPP_
 
-#include <type_traits>
 #include <iterator>
+#include <type_traits>
 
 #include "../../config.hpp"
 #include "../../detail/various.hpp"
@@ -39,7 +39,8 @@
 #include "../../block/block_radix_sort.hpp"
 #include "../../block/block_scan.hpp"
 #include "../../block/block_store_func.hpp"
-#include "../../thread/radix_key_codec.hpp"
+
+#include "ordered_block_id.hpp"
 
 BEGIN_ROCPRIM_NAMESPACE
 
@@ -63,7 +64,7 @@ void sort_warp_striped_to_striped(SortType sorter,
                                   unsigned int                     begin_bit,
                                   unsigned int                     end_bit)
 {
-    if ROCPRIM_IF_CONSTEXPR(Descending)
+    if constexpr(Descending)
     {
         sorter.sort_desc_warp_striped_to_striped(keys,
                                                  values,
@@ -92,8 +93,8 @@ void sort_warp_striped_to_striped(SortType sorter,
                                   unsigned int                     begin_bit,
                                   unsigned int                     end_bit)
 {
-    (void) values;
-    if ROCPRIM_IF_CONSTEXPR(Descending)
+    (void)values;
+    if constexpr(Descending)
     {
         sorter.sort_desc_warp_striped_to_striped(keys, storage, begin_bit, end_bit, decomposer);
     }
@@ -103,17 +104,15 @@ void sort_warp_striped_to_striped(SortType sorter,
     }
 }
 
-template<
-    unsigned int WarpSize,
-    unsigned int BlockSize,
-    unsigned int ItemsPerThread,
-    unsigned int RadixBits,
-    bool Descending
->
+template<unsigned int WarpSize,
+         unsigned int BlockSize,
+         unsigned int ItemsPerThread,
+         unsigned int RadixBits,
+         bool         Descending>
 struct radix_digit_count_helper
 {
     static constexpr unsigned int radix_size     = 1 << RadixBits;
-    static constexpr unsigned int warp_size = WarpSize;
+    static constexpr unsigned int warp_size      = WarpSize;
     static constexpr unsigned int atomic_stripes = 4;
     static constexpr unsigned int counters       = radix_size * atomic_stripes;
 
@@ -131,32 +130,28 @@ struct radix_digit_count_helper
     {
         assert(BlockSize % ::rocprim::arch::wavefront::size() == 0);
     }
-     
+
     ROCPRIM_DEVICE ROCPRIM_INLINE
     unsigned int get_counter(const unsigned stripe, const unsigned int digit)
     {
         return digit * atomic_stripes + stripe;
     }
 
-    template<
-        bool IsFull = false,
-        class KeysInputIterator,
-        class Offset
-    >
+    template<bool IsFull = false, class KeysInputIterator, class Offset>
     ROCPRIM_DEVICE ROCPRIM_INLINE
     void count_digits(KeysInputIterator keys_input,
-                      Offset begin_offset,
-                      Offset end_offset,
-                      unsigned int bit,
-                      unsigned int current_radix_bits,
-                      storage_type& storage,
-                      unsigned int& digit_count)  // i-th thread will get i-th digit's value
+                      Offset            begin_offset,
+                      Offset            end_offset,
+                      unsigned int      bit,
+                      unsigned int      current_radix_bits,
+                      storage_type&     storage,
+                      unsigned int&     digit_count) // i-th thread will get i-th digit's value
     {
         constexpr unsigned int items_per_block = BlockSize * ItemsPerThread;
 
         using key_type = typename std::iterator_traits<KeysInputIterator>::value_type;
-
-        using key_codec    = ::rocprim::radix_key_codec<key_type, Descending>;
+        using key_codec
+            = decltype(::rocprim::traits::get<key_type>().template radix_key_codec<Descending>());
         using bit_key_type = typename key_codec::bit_key_type;
 
         const unsigned int flat_id = ::rocprim::detail::block_thread_id<0>();
@@ -176,9 +171,10 @@ struct radix_digit_count_helper
 
         ::rocprim::syncthreads();
 
-        for(Offset block_offset = begin_offset; block_offset < end_offset; block_offset += items_per_block)
+        for(Offset block_offset = begin_offset; block_offset < end_offset;
+            block_offset += items_per_block)
         {
-            key_type keys[ItemsPerThread];
+            key_type     keys[ItemsPerThread];
             unsigned int valid_count;
             // Use loading into a striped arrangement because an order of items is irrelevant,
             // only totals matter
@@ -190,14 +186,18 @@ struct radix_digit_count_helper
             else
             {
                 valid_count = end_offset - block_offset;
-                block_load_direct_striped<BlockSize>(flat_id, keys_input + block_offset, keys, valid_count);
+                block_load_direct_striped<BlockSize>(flat_id,
+                                                     keys_input + block_offset,
+                                                     keys,
+                                                     valid_count);
             }
 
             ROCPRIM_UNROLL
             for(unsigned int i = 0; i < ItemsPerThread; i++)
             {
                 const bit_key_type bit_key = key_codec::encode(keys[i]);
-                const unsigned int digit = key_codec::extract_digit(bit_key, bit, current_radix_bits);
+                const unsigned int digit
+                    = key_codec::extract_digit(bit_key, bit, current_radix_bits);
                 const unsigned int pos = i * BlockSize + flat_id;
 
                 if(IsFull || pos < valid_count)
@@ -222,18 +222,16 @@ struct radix_digit_count_helper
     }
 };
 
-template<
-    unsigned int BlockSize,
-    unsigned int ItemsPerThread,
-    bool Descending,
-    class Key,
-    class Value
->
+template<unsigned int BlockSize,
+         unsigned int ItemsPerThread,
+         bool         Descending,
+         class Key,
+         class Value>
 struct radix_sort_single_helper
 {
     static constexpr unsigned int items_per_block = BlockSize * ItemsPerThread;
 
-    using key_type = Key;
+    using key_type   = Key;
     using value_type = Value;
 
     using sort_type = ::rocprim::block_radix_sort<key_type, BlockSize, ItemsPerThread, value_type>;
@@ -250,32 +248,33 @@ struct radix_sort_single_helper
              class ValuesInputIterator,
              class ValuesOutputIterator,
              class Decomposer>
-    ROCPRIM_DEVICE ROCPRIM_INLINE void sort_single(KeysInputIterator    keys_input,
-                                                   KeysOutputIterator   keys_output,
-                                                   ValuesInputIterator  values_input,
-                                                   ValuesOutputIterator values_output,
-                                                   unsigned int         size,
-                                                   Decomposer           decomposer,
-                                                   unsigned int         bit,
-                                                   unsigned int         current_radix_bits,
-                                                   storage_type&        storage)
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    void sort_single(KeysInputIterator    keys_input,
+                     KeysOutputIterator   keys_output,
+                     ValuesInputIterator  values_input,
+                     ValuesOutputIterator values_output,
+                     unsigned int         size,
+                     Decomposer           decomposer,
+                     unsigned int         bit,
+                     unsigned int         current_radix_bits,
+                     storage_type&        storage)
     {
-        const unsigned int flat_id = ::rocprim::detail::block_thread_id<0>();
-        const unsigned int flat_block_id = ::rocprim::detail::block_id<0>();
+        const unsigned int flat_id             = ::rocprim::detail::block_thread_id<0>();
+        const unsigned int flat_block_id       = ::rocprim::detail::block_id<0>();
         const unsigned int block_offset        = flat_block_id * items_per_block;
         const bool         is_incomplete_block = flat_block_id == (size / items_per_block);
         const unsigned int valid_in_last_block = size - block_offset;
 
         using key_type = typename std::iterator_traits<KeysInputIterator>::value_type;
+        using key_codec
+            = decltype(::rocprim::traits::get<key_type>().template radix_key_codec<Descending>());
 
-        using key_codec = radix_key_codec<key_type, Descending>;
-
-        key_type keys[ItemsPerThread];
+        key_type   keys[ItemsPerThread];
         value_type values[ItemsPerThread];
         if(!is_incomplete_block)
         {
             block_load_direct_warp_striped(flat_id, keys_input + block_offset, keys);
-            if ROCPRIM_IF_CONSTEXPR(with_values)
+            if constexpr(with_values)
             {
                 block_load_direct_warp_striped(flat_id, values_input + block_offset, values);
             }
@@ -288,7 +287,7 @@ struct radix_sort_single_helper
                                            keys,
                                            valid_in_last_block,
                                            out_of_bounds);
-            if ROCPRIM_IF_CONSTEXPR(with_values)
+            if constexpr(with_values)
             {
                 block_load_direct_warp_striped(flat_id,
                                                values_input + block_offset,
@@ -309,7 +308,7 @@ struct radix_sort_single_helper
         if(!is_incomplete_block)
         {
             block_store_direct_striped<BlockSize>(flat_id, keys_output + block_offset, keys);
-            if ROCPRIM_IF_CONSTEXPR(with_values)
+            if constexpr(with_values)
             {
                 block_store_direct_striped<BlockSize>(flat_id,
                                                       values_output + block_offset,
@@ -322,7 +321,7 @@ struct radix_sort_single_helper
                                                   keys_output + block_offset,
                                                   keys,
                                                   valid_in_last_block);
-            if ROCPRIM_IF_CONSTEXPR(with_values)
+            if constexpr(with_values)
             {
                 block_store_direct_striped<BlockSize>(flat_id,
                                                       values_output + block_offset,
@@ -343,12 +342,13 @@ template<unsigned int BlockSize,
          block_radix_rank_algorithm RadixRankAlgorithm = block_radix_rank_algorithm::match>
 struct radix_sort_and_scatter_helper
 {
-    static constexpr unsigned int items_per_block = BlockSize * ItemsPerThread;
-    static constexpr unsigned int radix_size = 1 << RadixBits;
+    static constexpr unsigned int items_per_block   = BlockSize * ItemsPerThread;
+    static constexpr unsigned int radix_size        = 1 << RadixBits;
     static constexpr unsigned int digits_per_thread = 1;
     static constexpr bool         with_values = !std::is_same<Value, ::rocprim::empty_type>::value;
 
-    using key_codec       = radix_key_codec<Key, Descending>;
+    using key_codec
+        = decltype(::rocprim::traits::get<Key>().template radix_key_codec<Descending>());
     using radix_rank_type = ::rocprim::block_radix_rank<BlockSize, RadixBits, RadixRankAlgorithm>;
 
     static constexpr bool load_warp_striped
@@ -378,9 +378,9 @@ struct radix_sort_and_scatter_helper
              class ValuesInputIterator,
              class ValuesOutputIterator>
     ROCPRIM_DEVICE ROCPRIM_INLINE
-    void sort_and_scatter(KeysInputIterator keys_input,
-                          KeysOutputIterator keys_output,
-                          ValuesInputIterator values_input,
+    void sort_and_scatter(KeysInputIterator    keys_input,
+                          KeysOutputIterator   keys_output,
+                          ValuesInputIterator  values_input,
                           ValuesOutputIterator values_output,
                           Offset               begin_offset,
                           Offset               end_offset,
@@ -397,7 +397,8 @@ struct radix_sort_and_scatter_helper
             storage.digit_offsets[flat_id] = digit_start;
         }
 
-        for(Offset block_offset = begin_offset; block_offset < end_offset; block_offset += items_per_block)
+        for(Offset block_offset = begin_offset; block_offset < end_offset;
+            block_offset += items_per_block)
         {
             Key keys[ItemsPerThread];
 
@@ -405,7 +406,7 @@ struct radix_sort_and_scatter_helper
             if(IsFull || (block_offset + items_per_block <= end_offset))
             {
                 valid_items = items_per_block;
-                if ROCPRIM_IF_CONSTEXPR(load_warp_striped)
+                if constexpr(load_warp_striped)
                 {
                     block_load_direct_warp_striped(flat_id, keys_input + block_offset, keys);
                 }
@@ -424,7 +425,7 @@ struct radix_sort_and_scatter_helper
                 // it does not matter. It does cause the final digit offset to be increased past its end,
                 // but again this does not matter since this is the last iteration in which it will be used anyway.
                 const Key out_of_bounds = key_codec::get_out_of_bounds_key();
-                if ROCPRIM_IF_CONSTEXPR(load_warp_striped)
+                if constexpr(load_warp_striped)
                 {
                     block_load_direct_warp_striped(flat_id,
                                                    keys_input + block_offset,
@@ -495,12 +496,12 @@ struct radix_sort_and_scatter_helper
             }
 
             // Gather and scatter values if necessary
-            if ROCPRIM_IF_CONSTEXPR(with_values)
+            if constexpr(with_values)
             {
                 Value values[ItemsPerThread];
-                if ROCPRIM_IF_CONSTEXPR(IsFull)
+                if constexpr(IsFull)
                 {
-                    if ROCPRIM_IF_CONSTEXPR(load_warp_striped)
+                    if constexpr(load_warp_striped)
                     {
                         block_load_direct_warp_striped(flat_id,
                                                        values_input + block_offset,
@@ -513,7 +514,7 @@ struct radix_sort_and_scatter_helper
                 }
                 else
                 {
-                    if ROCPRIM_IF_CONSTEXPR(load_warp_striped)
+                    if constexpr(load_warp_striped)
                     {
                         block_load_direct_warp_striped(flat_id,
                                                        values_input + block_offset,
@@ -596,13 +597,11 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE void sort_single(KeysInputIterator    keys_i
                                                      unsigned int         bit,
                                                      unsigned int         current_radix_bits)
 {
-    using key_type = typename std::iterator_traits<KeysInputIterator>::value_type;
+    using key_type   = typename std::iterator_traits<KeysInputIterator>::value_type;
     using value_type = typename std::iterator_traits<ValuesInputIterator>::value_type;
 
-    using sort_single_helper = radix_sort_single_helper<
-        BlockSize, ItemsPerThread, Descending,
-        key_type, value_type
-    >;
+    using sort_single_helper
+        = radix_sort_single_helper<BlockSize, ItemsPerThread, Descending, key_type, value_type>;
 
     ROCPRIM_SHARED_MEMORY typename sort_single_helper::storage_type storage;
 
@@ -619,8 +618,8 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE void sort_single(KeysInputIterator    keys_i
 
 template<class T>
 ROCPRIM_DEVICE ROCPRIM_INLINE
-auto compare_nan_sensitive(const T& a, const T& b)
-    -> typename std::enable_if<rocprim::is_floating_point<T>::value, bool>::type
+auto compare_nan_sensitive(const T& a, const T& b) ->
+    typename std::enable_if<rocprim::is_floating_point<T>::value, bool>::type
 {
     // Beware: the performance of this function is extremely vulnerable to refactoring.
     // Always check benchmark_device_segmented_radix_sort and benchmark_device_radix_sort
@@ -644,7 +643,8 @@ auto compare_nan_sensitive(const T& a, const T& b)
 }
 
 template<class T>
-ROCPRIM_DEVICE auto compare_nan_sensitive(const T& a, const T& b) ->
+ROCPRIM_DEVICE
+auto compare_nan_sensitive(const T& a, const T& b) ->
     typename std::enable_if<!rocprim::is_floating_point<T>::value, bool>::type
 {
     return a > b;
@@ -656,7 +656,8 @@ struct radix_merge_compare;
 template<class T>
 struct radix_merge_compare<false, false, T, identity_decomposer>
 {
-    ROCPRIM_DEVICE bool operator()(const T& a, const T& b) const
+    ROCPRIM_DEVICE
+    bool operator()(const T& a, const T& b) const
     {
         return compare_nan_sensitive<T>(b, a);
     }
@@ -665,7 +666,8 @@ struct radix_merge_compare<false, false, T, identity_decomposer>
 template<class T>
 struct radix_merge_compare<true, false, T, identity_decomposer>
 {
-    ROCPRIM_DEVICE bool operator()(const T& a, const T& b) const
+    ROCPRIM_DEVICE
+    bool operator()(const T& a, const T& b) const
     {
         return compare_nan_sensitive<T>(a, b);
     }
@@ -682,13 +684,14 @@ struct radix_merge_compare<Descending, true, T, identity_decomposer>
     {
         T radix_mask_upper  = (T(1) << (current_radix_bits + start_bit)) - 1;
         T radix_mask_bottom = (T(1) << start_bit) - 1;
-        radix_mask = radix_mask_upper ^ radix_mask_bottom;
+        radix_mask          = radix_mask_upper ^ radix_mask_bottom;
     }
 
-    ROCPRIM_DEVICE bool operator()(const T& a, const T& b) const
+    ROCPRIM_DEVICE
+    bool operator()(const T& a, const T& b) const
     {
-        const T masked_key_a  = a & radix_mask;
-        const T masked_key_b  = b & radix_mask;
+        const T masked_key_a = a & radix_mask;
+        const T masked_key_b = b & radix_mask;
         return Descending ? masked_key_a > masked_key_b : masked_key_b > masked_key_a;
     }
 };
@@ -706,9 +709,11 @@ struct radix_merge_compare<Descending, true, T, Decomposer>
         : decomposer_(decomposer), start_bit_(start_bit), radix_bits_(current_radix_bits)
     {}
 
-    ROCPRIM_HOST_DEVICE bool operator()(T lhs, T rhs) const
+    ROCPRIM_HOST_DEVICE
+    bool operator()(T lhs, T rhs) const
     {
-        using codec_t = radix_key_codec<T, Descending>;
+        using codec_t
+            = decltype(::rocprim::traits::get<T>().template radix_key_codec<Descending>());
 
         // Encoding the values considers the ascending / descending nature of the sort
         codec_t::encode_inplace(lhs, decomposer_);
@@ -773,7 +778,8 @@ struct onesweep_histograms_helper
         = radix_size * max_digit_places * atomic_stripes;
 
     using counter_type = uint32_t;
-    using key_codec    = radix_key_codec<KeyType, Descending>;
+    using key_codec
+        = decltype(::rocprim::traits::get<KeyType>().template radix_key_codec<Descending>());
 
     struct storage_type
     {
@@ -787,8 +793,8 @@ struct onesweep_histograms_helper
         return (place * radix_size + digit) * atomic_stripes + stripe_index;
     }
 
-    ROCPRIM_DEVICE ROCPRIM_INLINE void clear_histogram(const unsigned int flat_id,
-                                                       storage_type&      storage)
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    void clear_histogram(const unsigned int flat_id, storage_type& storage)
     {
         for(unsigned int i = flat_id; i < histogram_counters; i += BlockSize)
         {
@@ -811,7 +817,7 @@ struct onesweep_histograms_helper
 
         KeyType keys[ItemsPerThread];
         // Load using a striped arrangement, the order doesn't matter here.
-        if ROCPRIM_IF_CONSTEXPR(IsFull)
+        if constexpr(IsFull)
         {
             block_load_direct_striped<BlockSize>(flat_id, keys_input, keys);
         }
@@ -832,7 +838,7 @@ struct onesweep_histograms_helper
             key_codec::encode_inplace(keys[i], decomposer);
         }
 
-        if ROCPRIM_IF_CONSTEXPR(AllBits)
+        if constexpr(AllBits)
         {
             ROCPRIM_UNROLL
             for(unsigned int i = 0; i < ItemsPerThread; ++i)
@@ -916,7 +922,7 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE void onesweep_histograms(KeysInputIterator
                                                              const unsigned int begin_bit,
                                                              const unsigned int end_bit)
 {
-    using key_type = typename std::iterator_traits<KeysInputIterator>::value_type;
+    using key_type          = typename std::iterator_traits<KeysInputIterator>::value_type;
     using count_helper_type = onesweep_histograms_helper<key_type,
                                                          BlockSize,
                                                          ItemsPerThread,
@@ -926,7 +932,7 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE void onesweep_histograms(KeysInputIterator
 
     constexpr unsigned int items_per_block = BlockSize * ItemsPerThread;
 
-    const Offset block_id     = ::rocprim::detail::block_id<0>();
+    const Offset block_id = ::rocprim::detail::block_id<0>();
     const Offset block_offset = block_id * ItemsPerThread * BlockSize;
 
     ROCPRIM_SHARED_MEMORY typename count_helper_type::storage_type storage;
@@ -968,7 +974,8 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE void onesweep_histograms(KeysInputIterator
 }
 
 template<unsigned int BlockSize, unsigned int RadixBits, class Offset>
-ROCPRIM_DEVICE void onesweep_scan_histograms(Offset* global_digit_offsets)
+ROCPRIM_DEVICE
+void onesweep_scan_histograms(Offset* global_digit_offsets)
 {
     using block_scan_type = block_scan<Offset, BlockSize>;
 
@@ -1013,23 +1020,27 @@ struct onesweep_lookback_state
         : state(static_cast<underlying_type>(status) | value)
     {}
 
-    ROCPRIM_DEVICE ROCPRIM_INLINE underlying_type value() const
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    underlying_type value() const
     {
         return this->state & value_mask;
     }
 
-    ROCPRIM_DEVICE ROCPRIM_INLINE prefix_flag status() const
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    prefix_flag status() const
     {
         return static_cast<prefix_flag>(this->state & status_mask);
     }
 
-    ROCPRIM_DEVICE ROCPRIM_INLINE static onesweep_lookback_state load(onesweep_lookback_state* ptr)
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    static onesweep_lookback_state load(onesweep_lookback_state* ptr)
     {
         underlying_type state = ::rocprim::detail::atomic_load(&ptr->state);
         return onesweep_lookback_state(state);
     }
 
-    ROCPRIM_DEVICE ROCPRIM_INLINE void store(onesweep_lookback_state* ptr) const
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    void store(onesweep_lookback_state* ptr) const
     {
         ::rocprim::detail::atomic_store(&ptr->state, this->state);
     }
@@ -1043,14 +1054,16 @@ template<class Key,
          unsigned int               RadixBits,
          bool                       Descending,
          block_radix_rank_algorithm RadixRankAlgorithm,
-         class Decomposer>
+         class Decomposer,
+         class BlockIdWrapper>
 struct onesweep_iteration_helper
 {
     static constexpr unsigned int radix_size      = 1u << RadixBits;
     static constexpr unsigned int items_per_block = BlockSize * ItemsPerThread;
     static constexpr bool         with_values = !std::is_same<Value, rocprim::empty_type>::value;
 
-    using key_codec       = radix_key_codec<Key, Descending>;
+    using key_codec
+        = decltype(::rocprim::traits::get<Key>().template radix_key_codec<Descending>());
     using radix_rank_type = ::rocprim::block_radix_rank<BlockSize, RadixBits, RadixRankAlgorithm>;
 
     static constexpr bool load_warp_striped
@@ -1058,7 +1071,7 @@ struct onesweep_iteration_helper
 
     static constexpr unsigned int digits_per_thread = radix_rank_type::digits_per_thread;
 
-    union storage_type_
+    union data_storage
     {
         typename radix_rank_type::storage_type rank;
         struct
@@ -1066,12 +1079,18 @@ struct onesweep_iteration_helper
             Offset global_digit_offsets[radix_size];
             union
             {
-                Key          ordered_block_keys[items_per_block];
-                Value        ordered_block_values[items_per_block];
+                Key   ordered_block_keys[items_per_block];
+                Value ordered_block_values[items_per_block];
             };
         };
     };
 
+    struct storage_type_
+    {
+        data_storage                          data;
+        typename BlockIdWrapper::storage_type ordered_bid;
+    };
+
     ROCPRIM_DETAIL_SUPPRESS_DEPRECATION_WITH_PUSH
     using storage_type = detail::raw_storage<storage_type_>;
     ROCPRIM_DETAIL_SUPPRESS_DEPRECATION_POP
@@ -1081,28 +1100,30 @@ struct onesweep_iteration_helper
              class KeysOutputIterator,
              class ValuesInputIterator,
              class ValuesOutputIterator>
-    ROCPRIM_DEVICE void onesweep(KeysInputIterator        keys_input,
-                                 KeysOutputIterator       keys_output,
-                                 ValuesInputIterator      values_input,
-                                 ValuesOutputIterator     values_output,
-                                 Offset*                  global_digit_offsets_in,
-                                 Offset*                  global_digit_offsets_out,
-                                 onesweep_lookback_state* lookback_states,
-                                 Decomposer               decomposer,
-                                 const unsigned int       bit,
-                                 const unsigned int       current_radix_bits,
-                                 const unsigned int       valid_items,
-                                 storage_type_&           storage)
+    ROCPRIM_DEVICE
+    void onesweep(KeysInputIterator        keys_input,
+                  KeysOutputIterator       keys_output,
+                  ValuesInputIterator      values_input,
+                  ValuesOutputIterator     values_output,
+                  Offset*                  global_digit_offsets_in,
+                  Offset*                  global_digit_offsets_out,
+                  onesweep_lookback_state* lookback_states,
+                  Decomposer               decomposer,
+                  const unsigned int       bit,
+                  const unsigned int       current_radix_bits,
+                  const unsigned int       valid_items,
+                  data_storage&            storage,
+                  unsigned int             ordered_bid)
     {
         const unsigned int flat_id      = ::rocprim::detail::block_thread_id<0>();
-        const unsigned int block_id     = ::rocprim::detail::block_id<0>();
+        const unsigned int block_id     = ordered_bid;
         const unsigned int block_offset = block_id * items_per_block;
 
         // Load keys into private memory, and encode them to unsigned integers.
         Key keys[ItemsPerThread];
-        if ROCPRIM_IF_CONSTEXPR(IsFull)
+        if constexpr(IsFull)
         {
-            if ROCPRIM_IF_CONSTEXPR(load_warp_striped)
+            if constexpr(load_warp_striped)
             {
                 block_load_direct_warp_striped(flat_id, keys_input + block_offset, keys);
             }
@@ -1120,7 +1141,7 @@ struct onesweep_iteration_helper
             // it does not matter. It does cause the final digit offset to be increased past its end,
             // but again this does not matter since this is the last iteration in which it will be used anyway.
             const Key out_of_bounds = key_codec::get_out_of_bounds_key(decomposer);
-            if ROCPRIM_IF_CONSTEXPR(load_warp_striped)
+            if constexpr(load_warp_striped)
             {
                 block_load_direct_warp_striped(flat_id,
                                                keys_input + block_offset,
@@ -1241,9 +1262,9 @@ struct onesweep_iteration_helper
         if(with_values)
         {
             Value values[ItemsPerThread];
-            if ROCPRIM_IF_CONSTEXPR(IsFull)
+            if constexpr(IsFull)
             {
-                if ROCPRIM_IF_CONSTEXPR(load_warp_striped)
+                if constexpr(load_warp_striped)
                 {
                     block_load_direct_warp_striped(flat_id, values_input + block_offset, values);
                 }
@@ -1254,7 +1275,7 @@ struct onesweep_iteration_helper
             }
             else
             {
-                if ROCPRIM_IF_CONSTEXPR(load_warp_striped)
+                if constexpr(load_warp_striped)
                 {
                     block_load_direct_warp_striped(flat_id,
                                                    values_input + block_offset,
@@ -1337,7 +1358,8 @@ template<unsigned int               BlockSize,
          class ValuesInputIterator,
          class ValuesOutputIterator,
          class Offset,
-         class Decomposer>
+         class Decomposer,
+         class BlockIdWrapper>
 ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE void
     onesweep_iteration(KeysInputIterator        keys_input,
                        KeysOutputIterator       keys_output,
@@ -1350,7 +1372,8 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE void
                        Decomposer               decomposer,
                        const unsigned int       bit,
                        const unsigned int       current_radix_bits,
-                       const unsigned int       full_blocks)
+                       const unsigned int       full_blocks,
+                       BlockIdWrapper           ordered_bid)
 {
     using key_type   = typename std::iterator_traits<KeysInputIterator>::value_type;
     using value_type = typename std::iterator_traits<ValuesInputIterator>::value_type;
@@ -1363,13 +1386,15 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE void
                                                                      RadixBits,
                                                                      Descending,
                                                                      RadixRankAlgorithm,
-                                                                     Decomposer>;
-
-    constexpr unsigned int items_per_block = BlockSize * ItemsPerThread;
-    const unsigned int     block_id        = ::rocprim::detail::block_id<0>();
+                                                                     Decomposer,
+                                                                     BlockIdWrapper>;
 
     ROCPRIM_SHARED_MEMORY typename onesweep_iteration_helper_type::storage_type storage;
 
+    constexpr unsigned int items_per_block = BlockSize * ItemsPerThread;
+    const unsigned int     thread_id       = ::rocprim::detail::block_thread_id<0>();
+    const unsigned int     block_id        = ordered_bid.get(thread_id, storage.get().ordered_bid);
+
     if(block_id < full_blocks)
     {
         onesweep_iteration_helper_type{}.template onesweep<true>(keys_input,
@@ -1383,7 +1408,8 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE void
                                                                  bit,
                                                                  current_radix_bits,
                                                                  items_per_block,
-                                                                 storage.get());
+                                                                 storage.get().data,
+                                                                 block_id);
     }
     else
     {
@@ -1399,7 +1425,8 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE void
                                                                   bit,
                                                                   current_radix_bits,
                                                                   valid_in_last_block,
-                                                                  storage.get());
+                                                                  storage.get().data,
+                                                                  block_id);
     }
 }
 
