@@ -21,23 +21,17 @@
 #ifndef ROCPRIM_DEVICE_DETAIL_DEVICE_SCAN_LOOKBACK_HPP_
 #define ROCPRIM_DEVICE_DETAIL_DEVICE_SCAN_LOOKBACK_HPP_
 
-#include <iterator>
-#include <type_traits>
-
-#include "../../detail/various.hpp"
-#include "../../functional.hpp"
-#include "../../intrinsics.hpp"
-#include "../../types.hpp"
+#include "device_config_helper.hpp"
+#include "device_scan_common.hpp"
+#include "lookback_scan_state.hpp"
+#include "ordered_block_id.hpp"
 
 #include "../../block/block_load.hpp"
 #include "../../block/block_scan.hpp"
 #include "../../block/block_store.hpp"
+#include "../../intrinsics/thread.hpp"
 
-#include "../../device/device_scan_config.hpp"
-
-#include "device_scan_common.hpp"
-#include "lookback_scan_state.hpp"
-#include "ordered_block_id.hpp"
+#include <type_traits>
 
 BEGIN_ROCPRIM_NAMESPACE
 
@@ -108,7 +102,8 @@ template<class ArchConfig,
          class OutputIterator,
          class BinaryFunction,
          class AccType,
-         class LookbackScanState>
+         class LookbackScanState,
+         class BlockIdWrapper>
 ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE auto lookback_scan_kernel_impl(InputIterator,
                                                                    OutputIterator,
                                                                    const size_t,
@@ -116,10 +111,11 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE auto lookback_scan_kernel_impl(InputIterator
                                                                    BinaryFunction,
                                                                    LookbackScanState,
                                                                    const unsigned int,
-                                                                   AccType* = nullptr,
-                                                                   AccType* = nullptr,
-                                                                   bool     = false,
-                                                                   bool     = false)
+                                                                   AccType*,
+                                                                   AccType*,
+                                                                   bool,
+                                                                   bool,
+                                                                   BlockIdWrapper)
     -> std::enable_if_t<!is_lookback_kernel_runnable<LookbackScanState>()>
 {
     // No need to build the kernel with sleep on a device that does not require it
@@ -133,7 +129,8 @@ template<class ArchConfig,
          class OutputIterator,
          class BinaryFunction,
          class AccType,
-         class LookbackScanState>
+         class LookbackScanState,
+         class BlockIdWrapper>
 ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE auto
     lookback_scan_kernel_impl(InputIterator      input,
                               OutputIterator     output,
@@ -142,10 +139,11 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE auto
                               BinaryFunction     scan_op,
                               LookbackScanState  scan_state,
                               const unsigned int number_of_blocks,
-                              AccType*           previous_last_element = nullptr,
-                              AccType*           new_last_element      = nullptr,
-                              bool               override_first_value  = false,
-                              bool               save_last_value       = false)
+                              AccType*           previous_last_element,
+                              AccType*           new_last_element,
+                              bool               override_first_value,
+                              bool               save_last_value,
+                              BlockIdWrapper     ordered_bid)
         -> std::enable_if_t<is_lookback_kernel_runnable<LookbackScanState>()>
 {
     static_assert(std::is_same<AccType, typename LookbackScanState::value_type>::value,
@@ -167,13 +165,16 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE auto
 
     ROCPRIM_SHARED_MEMORY union
     {
+        typename BlockIdWrapper::storage_type   ordered_bid;
         typename block_load_type::storage_type  load;
         typename block_store_type::storage_type store;
         typename block_scan_type::storage_type  scan;
     } storage;
 
     const auto         flat_block_thread_id = ::rocprim::detail::block_thread_id<0>();
-    const auto         flat_block_id        = ::rocprim::detail::block_id<0>();
+    const size_t       flat_block_id
+        = ordered_bid.get(flat_block_thread_id,
+                          storage.ordered_bid); // ::rocprim::detail::block_id<0>();
     const unsigned int block_offset         = flat_block_id * items_per_block;
     const auto         valid_in_last_block  = size - items_per_block * (number_of_blocks - 1);
 
