@@ -21,12 +21,12 @@
 #ifndef ROCPRIM_DEVICE_DEVICE_SELECT_HPP_
 #define ROCPRIM_DEVICE_DEVICE_SELECT_HPP_
 
-#include <type_traits>
 #include <iterator>
+#include <type_traits>
 
 #include "../config.hpp"
-#include "../detail/various.hpp"
 #include "../detail/binary_op_wrappers.hpp"
+#include "../detail/various.hpp"
 
 #include "../iterator/transform_iterator.hpp"
 
@@ -37,11 +37,6 @@ BEGIN_ROCPRIM_NAMESPACE
 /// \addtogroup devicemodule
 /// @{
 
-namespace detail
-{
-
-} // end detail namespace
-
 /// \brief Parallel select primitive for device level using range of flags.
 ///
 /// Performs a device-wide selection based on input \p flags. If a value from \p input
@@ -58,27 +53,30 @@ namespace detail
 /// * Range specified by \p selected_count_output must have at least 1 element.
 /// * Values of \p flag range should be implicitly convertible to `bool` type.
 ///
-/// \tparam Config - [optional] Configuration of the primitive, must be `default_config` or `select_config`.
-/// \tparam InputIterator - random-access iterator type of the input range. It can be
+/// \tparam Config [optional] Configuration of the primitive, must be `default_config` or `select_config`.
+/// \tparam InputIterator random-access iterator type of the input range. It can be
 /// a simple pointer type.
-/// \tparam FlagIterator - random-access iterator type of the flag range. It can be
+/// \tparam FlagIterator random-access iterator type of the flag range. It can be
 /// a simple pointer type.
-/// \tparam OutputIterator - random-access iterator type of the output range. It can be
+/// \tparam OutputIterator random-access iterator type of the output range. It can be
 /// a simple pointer type.
-/// \tparam SelectedCountOutputIterator - random-access iterator type of the selected_count_output
+/// \tparam SelectedCountOutputIterator random-access iterator type of the selected_count_output
 /// value. It can be a simple pointer type.
+/// \tparam UsingOrderedBlockId If true, uses an atomic counter to assign block id instead of natural
+/// blockIdx-based ordering.  Can increase performance on MI3xx architectures when using streams. The
+/// default is false.
 ///
-/// \param [in] temporary_storage - pointer to a device-accessible temporary storage. When
+/// \param [in] temporary_storage pointer to a device-accessible temporary storage. When
 /// a null pointer is passed, the required allocation size (in bytes) is written to
 /// \p storage_size and function returns without performing the select operation.
-/// \param [in,out] storage_size - reference to a size (in bytes) of \p temporary_storage.
-/// \param [in] input - iterator to the first element in the range to select values from.
-/// \param [in] flags - iterator to the selection flag corresponding to the first element from \p input range.
-/// \param [out] output - iterator to the first element in the output range.
-/// \param [out] selected_count_output - iterator to the total number of selected values (length of \p output).
-/// \param [in] size - number of element in the input range.
-/// \param [in] stream - [optional] HIP stream object. The default is \p 0 (default stream).
-/// \param [in] debug_synchronous - [optional] If true, synchronization after every kernel
+/// \param [in,out] storage_size reference to a size (in bytes) of \p temporary_storage.
+/// \param [in] input iterator to the first element in the range to select values from.
+/// \param [in] flags iterator to the selection flag corresponding to the first element from \p input range.
+/// \param [out] output iterator to the first element in the output range.
+/// \param [out] selected_count_output iterator to the total number of selected values (length of \p output).
+/// \param [in] size number of element in the input range.
+/// \param [in] stream [optional] HIP stream object. The default is \p 0 (default stream).
+/// \param [in] debug_synchronous [optional] If true, synchronization after every kernel
 /// launch is forced in order to check for errors. The default value is \p false.
 ///
 /// \par Example
@@ -139,8 +137,8 @@ inline hipError_t select(void*                       temporary_storage,
     // Dummy unary predicate
     using unary_predicate_type = ::rocprim::empty_type;
     // Dummy inequality operation
-    using inequality_op_type = ::rocprim::empty_type;
-    using offset_type = unsigned int;
+    using inequality_op_type             = ::rocprim::empty_type;
+    using offset_type                    = size_t;
     rocprim::empty_type* const no_values = nullptr; // key only
 
     using output_key_iterator_tuple = tuple<OutputIterator, ::rocprim::empty_type>;
@@ -181,24 +179,27 @@ inline hipError_t select(void*                       temporary_storage,
 /// values can be copied into it.
 /// * Range specified by \p selected_count_output must have at least 1 element.
 ///
-/// \tparam Config - [optional] Configuration of the primitive, must be `default_config` or `select_config`.
-/// \tparam InputIterator - random-access iterator type of the input range. It can be
+/// \tparam Config [optional] Configuration of the primitive, must be `default_config` or `select_config`.
+/// \tparam InputIterator random-access iterator type of the input range. It can be
 /// a simple pointer type.
-/// \tparam OutputIterator - random-access iterator type of the output range. It can be
+/// \tparam OutputIterator random-access iterator type of the output range. It can be
 /// a simple pointer type.
-/// \tparam SelectedCountOutputIterator - random-access iterator type of the selected_count_output
+/// \tparam SelectedCountOutputIterator random-access iterator type of the selected_count_output
 /// value. It can be a simple pointer type.
-/// \tparam UnaryPredicate - type of a unary selection predicate.
+/// \tparam UnaryPredicate type of a unary selection predicate.
+/// \tparam UsingOrderedBlockId If true, uses an atomic counter to assign block id instead of natural
+/// blockIdx-based ordering.  Can increase performance on MI3xx architectures when using streams. The
+/// default is false.
 ///
-/// \param [in] temporary_storage - pointer to a device-accessible temporary storage. When
+/// \param [in] temporary_storage pointer to a device-accessible temporary storage. When
 /// a null pointer is passed, the required allocation size (in bytes) is written to
 /// \p storage_size and function returns without performing the select operation.
-/// \param [in,out] storage_size - reference to a size (in bytes) of \p temporary_storage.
-/// \param [in] input - iterator to the first element in the range to select values from.
-/// \param [out] output - iterator to the first element in the output range.
-/// \param [out] selected_count_output - iterator to the total number of selected values (length of \p output).
-/// \param [in] size - number of element in the input range.
-/// \param [in] predicate - unary function object that will be used for selecting values.
+/// \param [in,out] storage_size reference to a size (in bytes) of \p temporary_storage.
+/// \param [in] input iterator to the first element in the range to select values from.
+/// \param [out] output iterator to the first element in the output range.
+/// \param [out] selected_count_output iterator to the total number of selected values (length of \p output).
+/// \param [in] size number of element in the input range.
+/// \param [in] predicate unary function object that will be used for selecting values.
 /// The predicate must meet the C++ named requirement \p BinaryPredicate :
 ///   - The result of applying the predicate must be convertible to bool
 ///   - The predicate must accept const object arguments, with the same behavior regardless of
@@ -206,8 +207,8 @@ inline hipError_t select(void*                       temporary_storage,
 /// In practice, the signature of the function should be equivalent to the following:
 /// <tt>bool f(const T &a);</tt>. The signature does not need to have
 /// <tt>const &</tt>, but the function object must not modify the object passed to it.
-/// \param [in] stream - [optional] HIP stream object. The default is \p 0 (default stream).
-/// \param [in] debug_synchronous - [optional] If true, synchronization after every kernel
+/// \param [in] stream [optional] HIP stream object. The default is \p 0 (default stream).
+/// \param [in] debug_synchronous [optional] If true, synchronization after every kernel
 /// launch is forced in order to check for errors. The default value is \p false.
 ///
 /// \par Example
@@ -219,7 +220,7 @@ inline hipError_t select(void*                       temporary_storage,
 /// #include <rocprim/rocprim.hpp>
 ///
 /// auto predicate =
-///     [] __device__ (int a) -> bool
+///     [](int a) -> bool
 ///     {
 ///         return (a % 2) == 0;
 ///     };
@@ -269,11 +270,11 @@ inline hipError_t select(void*                       temporary_storage,
                          const bool                  debug_synchronous = false)
 {
     // Dummy flag type
-    using flag_type = ::rocprim::empty_type;
-    using offset_type = unsigned int;
-    flag_type * flags = nullptr;
+    using flag_type   = ::rocprim::empty_type;
+    using offset_type = size_t;
+    flag_type* flags  = nullptr;
     // Dummy inequality operation
-    using inequality_op_type = ::rocprim::empty_type;
+    using inequality_op_type             = ::rocprim::empty_type;
     rocprim::empty_type* const no_values = nullptr; // key only
 
     using output_key_iterator_tuple = tuple<OutputIterator, ::rocprim::empty_type>;
@@ -316,27 +317,30 @@ inline hipError_t select(void*                       temporary_storage,
 /// values can be copied into it.
 /// * Range specified by \p selected_count_output must have at least 1 element.
 ///
-/// \tparam Config - [optional] Configuration of the primitive, must be `default_config` or `select_config`.
-/// \tparam InputIterator - random-access iterator type of the input range. It can be
+/// \tparam Config [optional] Configuration of the primitive, must be `default_config` or `select_config`.
+/// \tparam InputIterator random-access iterator type of the input range. It can be
 /// a simple pointer type.
-/// \tparam FlagIterator - random-access iterator type of the flag range. It can be
+/// \tparam FlagIterator random-access iterator type of the flag range. It can be
 /// a simple pointer type.
-/// \tparam OutputIterator - random-access iterator type of the output range. It can be
+/// \tparam OutputIterator random-access iterator type of the output range. It can be
 /// a simple pointer type.
-/// \tparam SelectedCountOutputIterator - random-access iterator type of the selected_count_output
+/// \tparam SelectedCountOutputIterator random-access iterator type of the selected_count_output
 /// value. It can be a simple pointer type.
-/// \tparam UnaryPredicate - type of a unary selection predicate.
+/// \tparam UnaryPredicate type of a unary selection predicate.
+/// \tparam UsingOrderedBlockId If true, uses an atomic counter to assign block id instead of natural
+/// blockIdx-based ordering.  Can increase performance on MI3xx architectures when using streams. The
+/// default is false.
 ///
-/// \param [in] temporary_storage - pointer to a device-accessible temporary storage. When
+/// \param [in] temporary_storage pointer to a device-accessible temporary storage. When
 /// a null pointer is passed, the required allocation size (in bytes) is written to
 /// \p storage_size and function returns without performing the select operation.
-/// \param [in,out] storage_size - reference to a size (in bytes) of \p temporary_storage.
-/// \param [in] input - iterator to the first element in the range to select values from.
-/// \param [in] flags - iterator to the selection flag corresponding to the first element from \p input range.
-/// \param [out] output - iterator to the first element in the output range.
-/// \param [out] selected_count_output - iterator to the total number of selected values (length of \p output).
-/// \param [in] size - number of element in the input range.
-/// \param [in] predicate - unary function object that will be used for selecting flags.
+/// \param [in,out] storage_size reference to a size (in bytes) of \p temporary_storage.
+/// \param [in] input iterator to the first element in the range to select values from.
+/// \param [in] flags iterator to the selection flag corresponding to the first element from \p input range.
+/// \param [out] output iterator to the first element in the output range.
+/// \param [out] selected_count_output iterator to the total number of selected values (length of \p output).
+/// \param [in] size number of element in the input range.
+/// \param [in] predicate unary function object that will be used for selecting flags.
 /// The predicate must meet the C++ named requirement \p BinaryPredicate:
 ///   - The result of applying the predicate must be convertible to bool.
 ///   - The predicate must accept const object arguments, with the same behavior regardless of
@@ -344,8 +348,8 @@ inline hipError_t select(void*                       temporary_storage,
 /// In practice, the signature of the function should be equivalent to the following:
 /// <tt>bool f(const T &a);</tt>. The signature does not need to have
 /// <tt>const &</tt>, but the function object must not modify the object passed to it.
-/// \param [in] stream - [optional] HIP stream object. The default is \p 0 (default stream).
-/// \param [in] debug_synchronous - [optional] If true, synchronization after every kernel
+/// \param [in] stream [optional] HIP stream object. The default is \p 0 (default stream).
+/// \param [in] debug_synchronous [optional] If true, synchronization after every kernel
 /// launch is forced in order to check for errors. The default value is \p false.
 ///
 /// \par Example
@@ -357,7 +361,7 @@ inline hipError_t select(void*                       temporary_storage,
 /// #include <rocprim/rocprim.hpp>
 ///
 /// auto predicate =
-///     [] __device__ (int a) -> bool
+///     [](int a) -> bool
 ///     {
 ///         return (a % 2) == 0;
 ///     };
@@ -411,7 +415,7 @@ inline hipError_t select(void*                       temporary_storage,
 {
     // Dummy inequality operation
     using inequality_op_type             = ::rocprim::empty_type;
-    using offset_type                    = unsigned int;
+    using offset_type                    = size_t;
     rocprim::empty_type* const no_values = nullptr; // key only
 
     using output_key_iterator_tuple = tuple<OutputIterator, ::rocprim::empty_type>;
@@ -453,29 +457,32 @@ inline hipError_t select(void*                       temporary_storage,
 /// * By default <tt>InputIterator::value_type</tt>'s equality operator is used to check
 /// if elements are equivalent.
 ///
-/// \tparam Config - [optional] Configuration of the primitive, must be `default_config` or `select_config`.
-/// \tparam InputIterator - random-access iterator type of the input range. It can be
+/// \tparam Config [optional] Configuration of the primitive, must be `default_config` or `select_config`.
+/// \tparam InputIterator random-access iterator type of the input range. It can be
 /// a simple pointer type.
-/// \tparam OutputIterator - random-access iterator type of the output range. It can be
+/// \tparam OutputIterator random-access iterator type of the output range. It can be
 /// a simple pointer type.
-/// \tparam UniqueCountOutputIterator - random-access iterator type of the unique_count_output
+/// \tparam UniqueCountOutputIterator random-access iterator type of the unique_count_output
 /// value used to return number of unique values. It can be a simple pointer type.
-/// \tparam EqualityOp - type of an binary operator used to compare values for equality.
+/// \tparam EqualityOp type of an binary operator used to compare values for equality.
+/// \tparam UsingOrderedBlockId If true, uses an atomic counter to assign block id instead of natural
+/// blockIdx-based ordering.  Can increase performance on MI3xx architectures when using streams. The
+/// default is false.
 ///
-/// \param [in] temporary_storage - pointer to a device-accessible temporary storage. When
+/// \param [in] temporary_storage pointer to a device-accessible temporary storage. When
 /// a null pointer is passed, the required allocation size (in bytes) is written to
 /// \p storage_size and function returns without performing the unique operation.
-/// \param [in,out] storage_size - reference to a size (in bytes) of \p temporary_storage.
-/// \param [in] input - iterator to the first element in the range to select values from.
-/// \param [out] output - iterator to the first element in the output range.
-/// \param [out] unique_count_output - iterator to the total number of selected values (length of \p output).
-/// \param [in] size - number of element in the input range.
-/// \param [in] equality_op - [optional] binary function object used to compare input values for equality.
+/// \param [in,out] storage_size reference to a size (in bytes) of \p temporary_storage.
+/// \param [in] input iterator to the first element in the range to select values from.
+/// \param [out] output iterator to the first element in the output range.
+/// \param [out] unique_count_output iterator to the total number of selected values (length of \p output).
+/// \param [in] size number of element in the input range.
+/// \param [in] equality_op [optional] binary function object used to compare input values for equality.
 /// The signature of the function should be equivalent to the following:
 /// <tt>bool equal_to(const T &a, const T &b);</tt>. The signature does not need to have
 /// <tt>const &</tt>, but function object must not modify the object passed to it.
-/// \param [in] stream - [optional] HIP stream object. The default is \p 0 (default stream).
-/// \param [in] debug_synchronous - [optional] If true, synchronization after every kernel
+/// \param [in] stream [optional] HIP stream object. The default is \p 0 (default stream).
+/// \param [in] debug_synchronous [optional] If true, synchronization after every kernel
 /// launch is forced in order to check for errors. The default value is \p false.
 ///
 /// \par Example
@@ -532,10 +539,10 @@ inline hipError_t unique(void*                     temporary_storage,
 {
     // Dummy unary predicate
     using unary_predicate_type = ::rocprim::empty_type;
-    using offset_type = unsigned int;
+    using offset_type          = unsigned int;
     // Dummy flag type
-    using flag_type = ::rocprim::empty_type;
-    const flag_type * flags = nullptr;
+    using flag_type                      = ::rocprim::empty_type;
+    const flag_type*           flags     = nullptr;
     rocprim::empty_type* const no_values = nullptr; // key only
 
     // Convert equality operator to inequality operator
@@ -581,35 +588,38 @@ inline hipError_t unique(void*                     temporary_storage,
 /// * By default <tt>InputIterator::value_type</tt>'s equality operator is used to check
 /// if elements are equivalent.
 ///
-/// \tparam Config - [optional] Configuration of the primitive, must be `default_config` or `select_config`.
-/// \tparam KeyIterator - random-access iterator type of the input key range. It can be
+/// \tparam Config [optional] Configuration of the primitive, must be `default_config` or `select_config`.
+/// \tparam KeyIterator random-access iterator type of the input key range. It can be
 /// a simple pointer type.
-/// \tparam ValueIterator - random-access iterator type of the input value range. It can be
+/// \tparam ValueIterator random-access iterator type of the input value range. It can be
 /// a simple pointer type.
-/// \tparam OutputKeyIterator - random-access iterator type of the output key range. It can be
+/// \tparam OutputKeyIterator random-access iterator type of the output key range. It can be
 /// a simple pointer type.
-/// \tparam OutputValueIterator - random-access iterator type of the output value range. It can be
+/// \tparam OutputValueIterator random-access iterator type of the output value range. It can be
 /// a simple pointer type.
-/// \tparam UniqueCountOutputIterator - random-access iterator type of the unique_count_output
+/// \tparam UniqueCountOutputIterator random-access iterator type of the unique_count_output
 /// value used to return number of unique keys and values. It can be a simple pointer type.
-/// \tparam EqualityOp - type of an binary operator used to compare keys for equality.
+/// \tparam EqualityOp type of an binary operator used to compare keys for equality.
+/// \tparam UsingOrderedBlockId If true, uses an atomic counter to assign block id instead of natural
+/// blockIdx-based ordering.  Can increase performance on MI3xx architectures when using streams. The
+/// default is false.
 ///
-/// \param [in] temporary_storage - pointer to a device-accessible temporary storage. When
+/// \param [in] temporary_storage pointer to a device-accessible temporary storage. When
 /// a null pointer is passed, the required allocation size (in bytes) is written to
 /// \p storage_size and function returns without performing the unique operation.
-/// \param [in,out] storage_size - reference to a size (in bytes) of \p temporary_storage.
-/// \param [in] keys_input - iterator to the first element in the range to select keys from.
-/// \param [in] values_input - iterator to the first element in the range of values corresponding to keys
-/// \param [out] keys_output - iterator to the first element in the output key range.
-/// \param [out] values_output - iterator to the first element in the output value range.
-/// \param [out] unique_count_output - iterator to the total number of selected values (length of \p output).
-/// \param [in] size - number of element in the input range.
-/// \param [in] equality_op - [optional] binary function object used to compare input values for equality.
+/// \param [in,out] storage_size reference to a size (in bytes) of \p temporary_storage.
+/// \param [in] keys_input iterator to the first element in the range to select keys from.
+/// \param [in] values_input iterator to the first element in the range of values corresponding to keys
+/// \param [out] keys_output iterator to the first element in the output key range.
+/// \param [out] values_output iterator to the first element in the output value range.
+/// \param [out] unique_count_output iterator to the total number of selected values (length of \p output).
+/// \param [in] size number of element in the input range.
+/// \param [in] equality_op [optional] binary function object used to compare input values for equality.
 /// The signature of the function should be equivalent to the following:
 /// <tt>bool equal_to(const T &a, const T &b);</tt>. The signature does not need to have
 /// <tt>const &</tt>, but function object must not modify the object passed to it.
-/// \param [in] stream - [optional] HIP stream object. The default is \p 0 (default stream).
-/// \param [in] debug_synchronous - [optional] If true, synchronization after every kernel
+/// \param [in] stream [optional] HIP stream object. The default is \p 0 (default stream).
+/// \param [in] debug_synchronous [optional] If true, synchronization after every kernel
 /// launch is forced in order to check for errors. The default value is \p false.
 template<typename Config = default_config,
          typename KeyIterator,
