@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2018-2025 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -24,9 +24,11 @@
 #include <type_traits>
 
 #include "../config.hpp"
-#include "../functional.hpp"
 #include "../detail/various.hpp"
+#include "../functional.hpp"
 
+#include "detail/config/device_transform.hpp"
+#include "detail/config/device_transform_pointer.hpp"
 #include "detail/device_config_helper.hpp"
 
 /// \addtogroup primitivesmodule_deviceconfigs
@@ -37,12 +39,7 @@ BEGIN_ROCPRIM_NAMESPACE
 namespace detail
 {
 
-// device transform does not have config tuning
-template<unsigned int arch, class value_type>
-struct default_transform_config : default_transform_config_base<value_type>::type
-{};
-
-template<typename TransformConfig, typename>
+template<typename TransformConfig, typename, bool>
 struct wrapped_transform_config
 {
     static_assert(std::is_base_of<transform_config_tag, typename TransformConfig::tag>::value,
@@ -56,7 +53,18 @@ struct wrapped_transform_config
 };
 
 template<typename Value>
-struct wrapped_transform_config<default_config, Value>
+struct wrapped_transform_config<default_config, Value, true>
+{
+    template<target_arch Arch>
+    struct architecture_config
+    {
+        static constexpr transform_config_params params
+            = default_transform_pointer_config<static_cast<unsigned int>(Arch), Value>{};
+    };
+};
+
+template<typename Value>
+struct wrapped_transform_config<default_config, Value, false>
 {
     template<target_arch Arch>
     struct architecture_config
@@ -67,15 +75,20 @@ struct wrapped_transform_config<default_config, Value>
 };
 
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
-template<typename TransformConfig, typename Value>
+template<typename TransformConfig, typename Value, bool is_pointer>
 template<target_arch Arch>
 constexpr transform_config_params
-    wrapped_transform_config<TransformConfig, Value>::architecture_config<Arch>::params;
+    wrapped_transform_config<TransformConfig, Value, is_pointer>::architecture_config<Arch>::params;
 
 template<typename Value>
 template<target_arch Arch>
 constexpr transform_config_params
-    wrapped_transform_config<default_config, Value>::architecture_config<Arch>::params;
+    wrapped_transform_config<default_config, Value, true>::architecture_config<Arch>::params;
+
+template<typename Value>
+template<target_arch Arch>
+constexpr transform_config_params
+    wrapped_transform_config<default_config, Value, false>::architecture_config<Arch>::params;
 #endif // DOXYGEN_SHOULD_SKIP_THIS
 
 } // end namespace detail
