@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2018-2025 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,8 +21,10 @@
 #ifndef ROCPRIM_TYPES_TUPLE_HPP_
 #define ROCPRIM_TYPES_TUPLE_HPP_
 
+#include <functional>
 #include <tuple>
 #include <type_traits>
+#include <utility>
 
 #include "../config.hpp"
 #include "../detail/all_true.hpp"
@@ -797,8 +799,8 @@ struct tuple_less_than<0>
 
 /// \brief Equal to operator for tuples.
 ///
-/// \tparam TTypes... - the element types of \p lhs tuple.
-/// \tparam UTypes... - the element types of \p rhs tuple.
+/// \tparam TTypes... the element types of \p lhs tuple.
+/// \tparam UTypes... the element types of \p rhs tuple.
 ///
 /// Compares every element of the tuple lhs with the corresponding element
 /// of the tuple rhs, and returns \p true if all are equal.
@@ -823,8 +825,8 @@ bool operator==(const tuple<TTypes...>& lhs, const tuple<UTypes...>& rhs)
 
 /// \brief Not equal to operator for tuples.
 ///
-/// \tparam TTypes... - the element types of \p lhs tuple.
-/// \tparam UTypes... - the element types of \p rhs tuple.
+/// \tparam TTypes... the element types of \p lhs tuple.
+/// \tparam UTypes... the element types of \p rhs tuple.
 ///
 /// Compares every element of the tuple lhs with the corresponding element
 /// of the tuple rhs, and returns \p true if at least one of such pairs is
@@ -842,8 +844,8 @@ bool operator!=(const tuple<TTypes...>& lhs, const tuple<UTypes...>& rhs)
 
 /// \brief Less than operator for tuples.
 ///
-/// \tparam TTypes... - the element types of \p lhs tuple.
-/// \tparam UTypes... - the element types of \p rhs tuple.
+/// \tparam TTypes... the element types of \p lhs tuple.
+/// \tparam UTypes... the element types of \p rhs tuple.
 ///
 /// Compares lhs and rhs lexicographically.
 ///
@@ -868,8 +870,8 @@ bool operator<(const tuple<TTypes...>& lhs, const tuple<UTypes...>& rhs)
 
 /// \brief Greater than operator for tuples.
 ///
-/// \tparam TTypes... - the element types of \p lhs tuple.
-/// \tparam UTypes... - the element types of \p rhs tuple.
+/// \tparam TTypes... the element types of \p lhs tuple.
+/// \tparam UTypes... the element types of \p rhs tuple.
 ///
 /// Compares lhs and rhs lexicographically.
 ///
@@ -885,8 +887,8 @@ bool operator>(const tuple<TTypes...>& lhs, const tuple<UTypes...>& rhs)
 
 /// \brief Less than or equal to operator for tuples.
 ///
-/// \tparam TTypes... - the element types of \p lhs tuple.
-/// \tparam UTypes... - the element types of \p rhs tuple.
+/// \tparam TTypes... the element types of \p lhs tuple.
+/// \tparam UTypes... the element types of \p rhs tuple.
 ///
 /// Compares lhs and rhs lexicographically.
 ///
@@ -902,8 +904,8 @@ bool operator<=(const tuple<TTypes...>& lhs, const tuple<UTypes...>& rhs)
 
 /// \brief Greater than or equal to operator for tuples.
 ///
-/// \tparam TTypes... - the element types of \p lhs tuple.
-/// \tparam UTypes... - the element types of \p rhs tuple.
+/// \tparam TTypes... the element types of \p lhs tuple.
+/// \tparam UTypes... the element types of \p rhs tuple.
 ///
 /// Compares lhs and rhs lexicographically.
 ///
@@ -971,6 +973,15 @@ tuple_element_t<I, tuple<Types...>>&& get(tuple<Types...>&& t) noexcept
     return static_cast<value_type&&>(static_cast<type&>(t.base).get());
 }
 
+template<size_t I, class... Types>
+ROCPRIM_HOST_DEVICE
+inline tuple_element_t<I, tuple<Types...>>&& get(const tuple<Types...>&& t) noexcept
+{
+    using value_type = tuple_element_t<I, tuple<Types...>>;
+    using type       = detail::tuple_value<I, tuple_element_t<I, tuple<Types...>>>;
+    return static_cast<const value_type&&>(static_cast<const type&>(t.base).get());
+}
+
 // ////////////////////////
 // make_tuple
 // ////////////////////////
@@ -1010,7 +1021,7 @@ tuple<detail::make_tuple_return_t<Types>...> make_tuple(Types&&... args) noexcep
 /// unless \p std::decay<Ti>::type results in \p std::reference_wrapper<U> for some type U,
 /// in which case the deduced type is U&.
 ///
-/// \param args - zero or more arguments to create tuple from
+/// \param args zero or more arguments to create tuple from
 ///
 /// \see std::tuple
 template<class... Types>
@@ -1061,7 +1072,7 @@ const ignore_type ignore;
 /// \brief Creates a tuple of lvalue references to its arguments \p args or instances
 /// of \ref rocprim::ignore.
 ///
-/// \param args - zero or more input lvalue references used to create tuple
+/// \param args zero or more input lvalue references used to create tuple
 ///
 /// \see std::tie
 template<class... Types>
@@ -1071,6 +1082,32 @@ tuple<Types&...> tie(Types&... args) noexcept
     return ::rocprim::tuple<Types&...>(args...);
 }
 
+namespace detail
+{
+template<class Func, class Tuple, size_t... Indices>
+ROCPRIM_HOST_DEVICE ROCPRIM_INLINE
+auto apply_tuple(Func&& func, Tuple&& tuple, std::index_sequence<Indices...> /* indices */) noexcept
+{
+    return func(::rocprim::get<Indices>(std::forward<Tuple>(tuple))...);
+}
+} // namespace detail
+
+/// \brief Applies a function to a tuple.
+///
+/// \param func the function to apply.
+/// \param tuple the target tuple.
+///
+/// \see std::apply
+template<class Func, class Tuple>
+ROCPRIM_HOST_DEVICE ROCPRIM_INLINE
+auto apply(Func&& func, Tuple&& tuple) noexcept
+{
+    return ::rocprim::detail::apply_tuple(
+        std::forward<Func>(func),
+        std::forward<Tuple>(tuple),
+        std::make_index_sequence<rocprim::tuple_size<std::remove_reference_t<Tuple>>::value>{});
+}
+
 END_ROCPRIM_NAMESPACE
 
 /// @}
