@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2025 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -26,8 +26,8 @@
 #include "../config.hpp"
 #include "../detail/various.hpp"
 
-#include "../intrinsics.hpp"
 #include "../functional.hpp"
+#include "../intrinsics.hpp"
 #include "../types.hpp"
 
 #include "detail/warp_reduce_crosslane.hpp"
@@ -41,16 +41,16 @@ BEGIN_ROCPRIM_NAMESPACE
 namespace detail
 {
 
-// Select warp_reduce implementation based WarpSize
-template<class T, unsigned int WarpSize, bool UseAllReduce>
+// Select warp_reduce implementation based VirtualWaveSize
+template<class T, unsigned int VirtualWaveSize, bool UseAllReduce>
 struct select_warp_reduce_impl
 {
-    typedef typename std::conditional<
+    using type = typename std::conditional<
         // can we use crosslane (DPP or shuffle-based) implementation?
-        detail::is_warpsize_shuffleable<WarpSize>::value,
-        detail::warp_reduce_crosslane<T, WarpSize, UseAllReduce>, // yes
-        detail::warp_reduce_shared_mem<T, WarpSize, UseAllReduce> // no
-    >::type type;
+        detail::is_warpsize_shuffleable<VirtualWaveSize>::value,
+        detail::warp_reduce_crosslane<T, VirtualWaveSize, UseAllReduce>, // yes
+        detail::warp_reduce_shared_mem<T, VirtualWaveSize, UseAllReduce> // no
+        >::type;
 };
 
 } // end namespace detail
@@ -59,25 +59,25 @@ struct select_warp_reduce_impl
 /// for performing reduction operations on items partitioned across threads in a hardware
 /// warp.
 ///
-/// \tparam T - the input/output type.
-/// \tparam WarpSize - the size of logical warp size, which can be equal to or less than
+/// \tparam T the input/output type.
+/// \tparam VirtualWaveSize the size of logical warp size, which can be equal to or less than
 /// the size of hardware warp (see rocprim::arch::wavefront::min_size()). Reduce operations are performed
-/// separately within groups determined by WarpSize.
-/// \tparam UseAllReduce - input parameter to determine whether to broadcast final reduction
+/// separately within groups determined by VirtualWaveSize.
+/// \tparam UseAllReduce input parameter to determine whether to broadcast final reduction
 /// value to all threads (default is false).
 ///
 /// \par Overview
-/// * \p WarpSize must be equal to or less than the size of hardware warp (see
+/// * \p VirtualWaveSize must be equal to or less than the size of hardware warp (see
 /// rocprim::arch::wavefront::min_size()). If it is less, reduce is performed separately within groups
-/// determined by WarpSize. \n
-/// For example, if \p WarpSize is 4, hardware warp is 64, reduction will be performed in logical
+/// determined by VirtualWaveSize. \n
+/// For example, if \p VirtualWaveSize is 4, hardware warp is 64, reduction will be performed in logical
 /// warps grouped like this: `{ {0, 1, 2, 3}, {4, 5, 6, 7 }, ..., {60, 61, 62, 63} }`
 /// (thread is represented here by its id within hardware warp).
-/// * Logical warp is a group of \p WarpSize consecutive threads from the same hardware warp.
+/// * Logical warp is a group of \p VirtualWaveSize consecutive threads from the same hardware warp.
 /// * Supports non-commutative reduce operators. However, a reduce operator should be
 /// associative. When used with non-associative functions the results may be non-deterministic
 /// and/or vary in precision.
-/// * Number of threads executing warp_reduce's function must be a multiple of \p WarpSize;
+/// * Number of threads executing warp_reduce's function must be a multiple of \p VirtualWaveSize;
 /// * All threads from a logical warp must be in the same hardware warp.
 ///
 /// \par Examples
@@ -106,20 +106,25 @@ struct select_warp_reduce_impl
 /// }
 /// \endcode
 /// \endparblock
-template<
-    class T,
-    unsigned int WarpSize = arch::wavefront::min_size(),
-    bool UseAllReduce = false
->
+template<class T,
+         unsigned int            VirtualWaveSize = arch::wavefront::min_size(),
+         bool                    UseAllReduce    = false,
+         arch::wavefront::target TargetWaveSize  = arch::wavefront::get_target()>
 class warp_reduce
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
-    : private detail::select_warp_reduce_impl<T, WarpSize, UseAllReduce>::type
+    : private detail::select_warp_reduce_impl<T, VirtualWaveSize, UseAllReduce>::type
 #endif
 {
-    using base_type = typename detail::select_warp_reduce_impl<T, WarpSize, UseAllReduce>::type;
+    using base_type =
+        typename detail::select_warp_reduce_impl<T, VirtualWaveSize, UseAllReduce>::type;
 
-    // Check if WarpSize is valid for the targets
-    static_assert(WarpSize <= ROCPRIM_MAX_WARP_SIZE, "WarpSize can't be greater than hardware warp size.");
+    // Check if VirtualWaveSize is valid for the targets
+
+public:
+    ROCPRIM_INLINE ROCPRIM_HOST_DEVICE warp_reduce()
+    {
+        detail::check_virtual_wave_size<VirtualWaveSize>();
+    }
 
 public:
     /// \brief Struct used to allocate a temporary memory that is required for thread
@@ -134,13 +139,13 @@ public:
 
     /// \brief Performs reduction across threads in a logical warp.
     ///
-    /// \tparam BinaryFunction - type of binary function used for reduce. Default type
+    /// \tparam BinaryFunction type of binary function used for reduce. Default type
     /// is rocprim::plus<T>.
     ///
-    /// \param [in] input - thread input value.
-    /// \param [out] output - reference to a thread output value. May be aliased with \p input.
-    /// \param [in] storage - reference to a temporary storage object of type storage_type.
-    /// \param [in] reduce_op - binary operation function object that will be used for reduce.
+    /// \param [in] input thread input value.
+    /// \param [out] output reference to a thread output value. May be aliased with \p input.
+    /// \param [in] storage reference to a temporary storage object of type storage_type.
+    /// \param [in] reduce_op binary operation function object that will be used for reduce.
     /// The signature of the function should be equivalent to the following:
     /// <tt>T f(const T &a, const T &b);</tt>. The signature does not need to have
     /// <tt>const &</tt>, but function object must not modify the objects passed to it.
@@ -176,38 +181,52 @@ public:
     /// }
     /// \endcode
     /// \endparblock
-    template<class BinaryFunction = ::rocprim::plus<T>, unsigned int FunctionWarpSize = WarpSize>
-    ROCPRIM_DEVICE ROCPRIM_INLINE auto reduce(T              input,
-                                              T&             output,
-                                              storage_type&  storage,
-                                              BinaryFunction reduce_op = BinaryFunction()) ->
-        typename std::enable_if<(FunctionWarpSize <= arch::wavefront::min_size()), void>::type
+    template<class BinaryFunction          = ::rocprim::plus<T>,
+             unsigned int FunctionWarpSize = VirtualWaveSize>
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    auto reduce(T              input,
+                T&             output,
+                storage_type&  storage,
+                BinaryFunction reduce_op = BinaryFunction()) ->
+        typename std::enable_if<(FunctionWarpSize <= arch::wavefront::max_size()), void>::type
     {
+        if constexpr(TargetWaveSize == ::rocprim::arch::wavefront::target::dynamic)
+        {
+            if(VirtualWaveSize > ::rocprim::arch::wavefront::size())
+            {
+                ROCPRIM_PRINT_ERROR_ONCE(
+                    "Specified warp size exceeds current hardware supported warp "
+                    "size. Aborting warp sort.");
+                return;
+            }
+        }
         base_type::reduce(input, output, storage, reduce_op);
     }
 
     /// \brief Performs reduction across threads in a logical warp.
     /// Invalid Warp Size
-    template<class BinaryFunction = ::rocprim::plus<T>, unsigned int FunctionWarpSize = WarpSize>
-    ROCPRIM_DEVICE ROCPRIM_INLINE auto
-        reduce(T, T&, storage_type&, BinaryFunction reduce_op = BinaryFunction()) ->
-        typename std::enable_if<(FunctionWarpSize > arch::wavefront::min_size()), void>::type
+    template<class BinaryFunction          = ::rocprim::plus<T>,
+             unsigned int FunctionWarpSize = VirtualWaveSize>
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    auto reduce(T, T&, storage_type&, BinaryFunction reduce_op = BinaryFunction()) ->
+        typename std::enable_if<(FunctionWarpSize > arch::wavefront::max_size()), void>::type
     {
-        (void) reduce_op;
-        ROCPRIM_PRINT_ERROR_ONCE("Specified warp size exceeds current hardware supported warp size. Aborting warp sort.");
+        (void)reduce_op;
+        ROCPRIM_PRINT_ERROR_ONCE("Specified warp size exceeds current hardware supported warp "
+                                 "size. Aborting warp sort.");
         return;
     }
 
     /// \brief Performs reduction across threads in a logical warp.
     ///
-    /// \tparam BinaryFunction - type of binary function used for reduce. Default type
+    /// \tparam BinaryFunction type of binary function used for reduce. Default type
     /// is rocprim::plus<T>.
     ///
-    /// \param [in] input - thread input value.
-    /// \param [out] output - reference to a thread output value. May be aliased with \p input.
-    /// \param [in] valid_items - number of items that will be reduced in the warp.
-    /// \param [in] storage - reference to a temporary storage object of type storage_type.
-    /// \param [in] reduce_op - binary operation function object that will be used for reduce.
+    /// \param [in] input thread input value.
+    /// \param [out] output reference to a thread output value. May be aliased with \p input.
+    /// \param [in] valid_items number of items that will be reduced in the warp.
+    /// \param [in] storage reference to a temporary storage object of type storage_type.
+    /// \param [in] reduce_op binary operation function object that will be used for reduce.
     /// The signature of the function should be equivalent to the following:
     /// <tt>T f(const T &a, const T &b);</tt>. The signature does not need to have
     /// <tt>const &</tt>, but function object must not modify the objects passed to it.
@@ -244,40 +263,54 @@ public:
     /// }
     /// \endcode
     /// \endparblock
-    template<class BinaryFunction = ::rocprim::plus<T>, unsigned int FunctionWarpSize = WarpSize>
-    ROCPRIM_DEVICE ROCPRIM_INLINE auto reduce(T              input,
-                                              T&             output,
-                                              int            valid_items,
-                                              storage_type&  storage,
-                                              BinaryFunction reduce_op = BinaryFunction()) ->
-        typename std::enable_if<(FunctionWarpSize <= arch::wavefront::min_size()), void>::type
+    template<class BinaryFunction          = ::rocprim::plus<T>,
+             unsigned int FunctionWarpSize = VirtualWaveSize>
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    auto reduce(T              input,
+                T&             output,
+                int            valid_items,
+                storage_type&  storage,
+                BinaryFunction reduce_op = BinaryFunction()) ->
+        typename std::enable_if<(FunctionWarpSize <= arch::wavefront::max_size()), void>::type
     {
+        if constexpr(TargetWaveSize == ::rocprim::arch::wavefront::target::dynamic)
+        {
+            if(VirtualWaveSize > ::rocprim::arch::wavefront::size())
+            {
+                ROCPRIM_PRINT_ERROR_ONCE(
+                    "Specified warp size exceeds current hardware supported warp "
+                    "size. Aborting warp sort.");
+                return;
+            }
+        }
         base_type::reduce(input, output, valid_items, storage, reduce_op);
     }
 
     /// \brief Performs reduction across threads in a logical warp.
     /// Invalid Warp Size
-    template<class BinaryFunction = ::rocprim::plus<T>, unsigned int FunctionWarpSize = WarpSize>
-    ROCPRIM_DEVICE ROCPRIM_INLINE auto
-        reduce(T, T&, int, storage_type&, BinaryFunction reduce_op = BinaryFunction()) ->
-        typename std::enable_if<(FunctionWarpSize > arch::wavefront::min_size()), void>::type
+    template<class BinaryFunction          = ::rocprim::plus<T>,
+             unsigned int FunctionWarpSize = VirtualWaveSize>
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    auto reduce(T, T&, int, storage_type&, BinaryFunction reduce_op = BinaryFunction()) ->
+        typename std::enable_if<(FunctionWarpSize > arch::wavefront::max_size()), void>::type
     {
-        (void) reduce_op;
-        ROCPRIM_PRINT_ERROR_ONCE("Specified warp size exceeds current hardware supported warp size. Aborting warp sort.");
+        (void)reduce_op;
+        ROCPRIM_PRINT_ERROR_ONCE("Specified warp size exceeds current hardware supported warp "
+                                 "size. Aborting warp sort.");
         return;
     }
 
     /// \brief Performs head-segmented reduction across threads in a logical warp.
     ///
-    /// \tparam Flag - type of head flags. Must be contextually convertible to \p bool.
-    /// \tparam BinaryFunction - type of binary function used for reduce. Default type
+    /// \tparam Flag type of head flags. Must be contextually convertible to \p bool.
+    /// \tparam BinaryFunction type of binary function used for reduce. Default type
     /// is rocprim::plus<T>.
     ///
-    /// \param [in] input - thread input value.
-    /// \param [out] output - reference to a thread output value. May be aliased with \p input.
-    /// \param [in] flag - thread head flag, \p true flags mark beginnings of segments.
-    /// \param [in] storage - reference to a temporary storage object of type storage_type.
-    /// \param [in] reduce_op - binary operation function object that will be used for reduce.
+    /// \param [in] input thread input value.
+    /// \param [out] output reference to a thread output value. May be aliased with \p input.
+    /// \param [in] flag thread head flag, \p true flags mark beginnings of segments.
+    /// \param [in] storage reference to a temporary storage object of type storage_type.
+    /// \param [in] reduce_op binary operation function object that will be used for reduce.
     /// The signature of the function should be equivalent to the following:
     /// <tt>T f(const T &a, const T &b);</tt>. The signature does not need to have
     /// <tt>const &</tt>, but function object must not modify the objects passed to it.
@@ -287,15 +320,25 @@ public:
     /// or repurposed: \p __syncthreads() or \p rocprim::syncthreads().
     template<class Flag,
              class BinaryFunction          = ::rocprim::plus<T>,
-             unsigned int FunctionWarpSize = WarpSize>
-    ROCPRIM_DEVICE ROCPRIM_INLINE auto head_segmented_reduce(T              input,
-                                                             T&             output,
-                                                             Flag           flag,
-                                                             storage_type&  storage,
-                                                             BinaryFunction reduce_op
-                                                             = BinaryFunction()) ->
-        typename std::enable_if<(FunctionWarpSize <= arch::wavefront::min_size()), void>::type
+             unsigned int FunctionWarpSize = VirtualWaveSize>
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    auto head_segmented_reduce(T              input,
+                               T&             output,
+                               Flag           flag,
+                               storage_type&  storage,
+                               BinaryFunction reduce_op = BinaryFunction()) ->
+        typename std::enable_if<(FunctionWarpSize <= arch::wavefront::max_size()), void>::type
     {
+        if constexpr(TargetWaveSize == ::rocprim::arch::wavefront::target::dynamic)
+        {
+            if(VirtualWaveSize > ::rocprim::arch::wavefront::size())
+            {
+                ROCPRIM_PRINT_ERROR_ONCE(
+                    "Specified warp size exceeds current hardware supported warp "
+                    "size. Aborting warp sort.");
+                return;
+            }
+        }
         base_type::head_segmented_reduce(input, output, flag, storage, reduce_op);
     }
 
@@ -303,27 +346,29 @@ public:
     /// Invalid Warp Size
     template<class Flag,
              class BinaryFunction          = ::rocprim::plus<T>,
-             unsigned int FunctionWarpSize = WarpSize>
-    ROCPRIM_DEVICE ROCPRIM_INLINE auto head_segmented_reduce(
+             unsigned int FunctionWarpSize = VirtualWaveSize>
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    auto head_segmented_reduce(
         T, T&, Flag, storage_type&, BinaryFunction reduce_op = BinaryFunction()) ->
-        typename std::enable_if<(FunctionWarpSize > arch::wavefront::min_size()), void>::type
+        typename std::enable_if<(FunctionWarpSize > arch::wavefront::max_size()), void>::type
     {
-        (void) reduce_op;
-        ROCPRIM_PRINT_ERROR_ONCE("Specified warp size exceeds current hardware supported warp size. Aborting warp sort.");
+        (void)reduce_op;
+        ROCPRIM_PRINT_ERROR_ONCE("Specified warp size exceeds current hardware supported warp "
+                                 "size. Aborting warp sort.");
         return;
     }
 
     /// \brief Performs tail-segmented reduction across threads in a logical warp.
     ///
-    /// \tparam Flag - type of tail flags. Must be contextually convertible to \p bool.
-    /// \tparam BinaryFunction - type of binary function used for reduce. Default type
+    /// \tparam Flag type of tail flags. Must be contextually convertible to \p bool.
+    /// \tparam BinaryFunction type of binary function used for reduce. Default type
     /// is rocprim::plus<T>.
     ///
-    /// \param [in] input - thread input value.
-    /// \param [out] output - reference to a thread output value. May be aliased with \p input.
-    /// \param [in] flag - thread tail flag, \p true flags mark ends of segments.
-    /// \param [in] storage - reference to a temporary storage object of type storage_type.
-    /// \param [in] reduce_op - binary operation function object that will be used for reduce.
+    /// \param [in] input thread input value.
+    /// \param [out] output reference to a thread output value. May be aliased with \p input.
+    /// \param [in] flag thread tail flag, \p true flags mark ends of segments.
+    /// \param [in] storage reference to a temporary storage object of type storage_type.
+    /// \param [in] reduce_op binary operation function object that will be used for reduce.
     /// The signature of the function should be equivalent to the following:
     /// <tt>T f(const T &a, const T &b);</tt>. The signature does not need to have
     /// <tt>const &</tt>, but function object must not modify the objects passed to it.
@@ -333,15 +378,25 @@ public:
     /// or repurposed: \p __syncthreads() or \p rocprim::syncthreads().
     template<class Flag,
              class BinaryFunction          = ::rocprim::plus<T>,
-             unsigned int FunctionWarpSize = WarpSize>
-    ROCPRIM_DEVICE ROCPRIM_INLINE auto tail_segmented_reduce(T              input,
-                                                             T&             output,
-                                                             Flag           flag,
-                                                             storage_type&  storage,
-                                                             BinaryFunction reduce_op
-                                                             = BinaryFunction()) ->
-        typename std::enable_if<(FunctionWarpSize <= arch::wavefront::min_size()), void>::type
+             unsigned int FunctionWarpSize = VirtualWaveSize>
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    auto tail_segmented_reduce(T              input,
+                               T&             output,
+                               Flag           flag,
+                               storage_type&  storage,
+                               BinaryFunction reduce_op = BinaryFunction()) ->
+        typename std::enable_if<(FunctionWarpSize <= arch::wavefront::max_size()), void>::type
     {
+        if constexpr(TargetWaveSize == ::rocprim::arch::wavefront::target::dynamic)
+        {
+            if(VirtualWaveSize > ::rocprim::arch::wavefront::size())
+            {
+                ROCPRIM_PRINT_ERROR_ONCE(
+                    "Specified warp size exceeds current hardware supported warp "
+                    "size. Aborting warp sort.");
+                return;
+            }
+        }
         base_type::tail_segmented_reduce(input, output, flag, storage, reduce_op);
     }
 
@@ -349,13 +404,15 @@ public:
     /// Invalid Warp Size
     template<class Flag,
              class BinaryFunction          = ::rocprim::plus<T>,
-             unsigned int FunctionWarpSize = WarpSize>
-    ROCPRIM_DEVICE ROCPRIM_INLINE auto tail_segmented_reduce(
+             unsigned int FunctionWarpSize = VirtualWaveSize>
+    ROCPRIM_DEVICE ROCPRIM_INLINE
+    auto tail_segmented_reduce(
         T, T&, Flag, storage_type&, BinaryFunction reduce_op = BinaryFunction()) ->
-        typename std::enable_if<(FunctionWarpSize > arch::wavefront::min_size()), void>::type
+        typename std::enable_if<(FunctionWarpSize > arch::wavefront::max_size()), void>::type
     {
-        (void) reduce_op;
-        ROCPRIM_PRINT_ERROR_ONCE("Specified warp size exceeds current hardware supported warp size. Aborting warp sort.");
+        (void)reduce_op;
+        ROCPRIM_PRINT_ERROR_ONCE("Specified warp size exceeds current hardware supported warp "
+                                 "size. Aborting warp sort.");
         return;
     }
 };
